# coding: utf-8

# flake8: noqa
"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi_client.models.campaign.account_migration_statuses_info import AccountMigrationStatusesInfo
from openapi_client.models.campaign.account_negative_keyword_list import AccountNegativeKeywordList
from openapi_client.models.campaign.account_placement_exclusion_list import AccountPlacementExclusionList
from openapi_client.models.campaign.account_placement_inclusion_list import AccountPlacementInclusionList
from openapi_client.models.campaign.account_property import AccountProperty
from openapi_client.models.campaign.account_property_name import AccountPropertyName
from openapi_client.models.campaign.action_ad_extension import ActionAdExtension
from openapi_client.models.campaign.action_ad_extension_action_type import ActionAdExtensionActionType
from openapi_client.models.campaign.ad import Ad
from openapi_client.models.campaign.ad_additional_field import AdAdditionalField
from openapi_client.models.campaign.ad_api_error import AdApiError
from openapi_client.models.campaign.ad_api_fault_detail import AdApiFaultDetail
from openapi_client.models.campaign.ad_center_authentication_fault import AdCenterAuthenticationFault
from openapi_client.models.campaign.ad_center_authorization_fault import AdCenterAuthorizationFault
from openapi_client.models.campaign.ad_center_fault import AdCenterFault
from openapi_client.models.campaign.ad_center_internal_error_fault import AdCenterInternalErrorFault
from openapi_client.models.campaign.ad_center_system_fault import AdCenterSystemFault
from openapi_client.models.campaign.ad_editorial_status import AdEditorialStatus
from openapi_client.models.campaign.ad_extension import AdExtension
from openapi_client.models.campaign.ad_extension_additional_field import AdExtensionAdditionalField
from openapi_client.models.campaign.ad_extension_association import AdExtensionAssociation
from openapi_client.models.campaign.ad_extension_association_collection import AdExtensionAssociationCollection
from openapi_client.models.campaign.ad_extension_editorial_reason import AdExtensionEditorialReason
from openapi_client.models.campaign.ad_extension_editorial_reason_collection import AdExtensionEditorialReasonCollection
from openapi_client.models.campaign.ad_extension_editorial_status import AdExtensionEditorialStatus
from openapi_client.models.campaign.ad_extension_header_type import AdExtensionHeaderType
from openapi_client.models.campaign.ad_extension_id_to_entity_id_association import AdExtensionIdToEntityIdAssociation
from openapi_client.models.campaign.ad_extension_identity import AdExtensionIdentity
from openapi_client.models.campaign.ad_extension_status import AdExtensionStatus
from openapi_client.models.campaign.ad_extensions_type_filter import AdExtensionsTypeFilter
from openapi_client.models.campaign.ad_group import AdGroup
from openapi_client.models.campaign.ad_group_additional_field import AdGroupAdditionalField
from openapi_client.models.campaign.ad_group_criterion import AdGroupCriterion
from openapi_client.models.campaign.ad_group_criterion_action import AdGroupCriterionAction
from openapi_client.models.campaign.ad_group_criterion_editorial_status import AdGroupCriterionEditorialStatus
from openapi_client.models.campaign.ad_group_criterion_status import AdGroupCriterionStatus
from openapi_client.models.campaign.ad_group_criterion_type import AdGroupCriterionType
from openapi_client.models.campaign.ad_group_negative_sites import AdGroupNegativeSites
from openapi_client.models.campaign.ad_group_privacy_status import AdGroupPrivacyStatus
from openapi_client.models.campaign.ad_group_status import AdGroupStatus
from openapi_client.models.campaign.ad_recommendation_additional_field import AdRecommendationAdditionalField
from openapi_client.models.campaign.ad_recommendation_customized_property import AdRecommendationCustomizedProperty
from openapi_client.models.campaign.ad_recommendation_image_asset_property import AdRecommendationImageAssetProperty
from openapi_client.models.campaign.ad_recommendation_image_field import AdRecommendationImageField
from openapi_client.models.campaign.ad_recommendation_image_refine_operation import AdRecommendationImageRefineOperation
from openapi_client.models.campaign.ad_recommendation_image_refine_type import AdRecommendationImageRefineType
from openapi_client.models.campaign.ad_recommendation_image_suggestion import AdRecommendationImageSuggestion
from openapi_client.models.campaign.ad_recommendation_image_suggestion_metadata import AdRecommendationImageSuggestionMetadata
from openapi_client.models.campaign.ad_recommendation_media_refine_result import AdRecommendationMediaRefineResult
from openapi_client.models.campaign.ad_recommendation_refined_media import AdRecommendationRefinedMedia
from openapi_client.models.campaign.ad_recommendation_text_asset_property import AdRecommendationTextAssetProperty
from openapi_client.models.campaign.ad_recommendation_text_field import AdRecommendationTextField
from openapi_client.models.campaign.ad_recommendation_text_refine_operation import AdRecommendationTextRefineOperation
from openapi_client.models.campaign.ad_recommendation_text_refine_result import AdRecommendationTextRefineResult
from openapi_client.models.campaign.ad_recommendation_text_tone import AdRecommendationTextTone
from openapi_client.models.campaign.ad_recommendation_video_suggestion import AdRecommendationVideoSuggestion
from openapi_client.models.campaign.ad_recommendation_video_type import AdRecommendationVideoType
from openapi_client.models.campaign.ad_rotation import AdRotation
from openapi_client.models.campaign.ad_rotation_type import AdRotationType
from openapi_client.models.campaign.ad_status import AdStatus
from openapi_client.models.campaign.ad_sub_type import AdSubType
from openapi_client.models.campaign.ad_type import AdType
from openapi_client.models.campaign.add_ad_extensions_request import AddAdExtensionsRequest
from openapi_client.models.campaign.add_ad_extensions_response import AddAdExtensionsResponse
from openapi_client.models.campaign.add_ad_group_criterions_request import AddAdGroupCriterionsRequest
from openapi_client.models.campaign.add_ad_group_criterions_response import AddAdGroupCriterionsResponse
from openapi_client.models.campaign.add_ad_groups_request import AddAdGroupsRequest
from openapi_client.models.campaign.add_ad_groups_response import AddAdGroupsResponse
from openapi_client.models.campaign.add_ads_request import AddAdsRequest
from openapi_client.models.campaign.add_ads_response import AddAdsResponse
from openapi_client.models.campaign.add_asset_groups_request import AddAssetGroupsRequest
from openapi_client.models.campaign.add_asset_groups_response import AddAssetGroupsResponse
from openapi_client.models.campaign.add_audience_groups_request import AddAudienceGroupsRequest
from openapi_client.models.campaign.add_audience_groups_response import AddAudienceGroupsResponse
from openapi_client.models.campaign.add_audiences_request import AddAudiencesRequest
from openapi_client.models.campaign.add_audiences_response import AddAudiencesResponse
from openapi_client.models.campaign.add_bid_strategies_request import AddBidStrategiesRequest
from openapi_client.models.campaign.add_bid_strategies_response import AddBidStrategiesResponse
from openapi_client.models.campaign.add_brand_kits_request import AddBrandKitsRequest
from openapi_client.models.campaign.add_brand_kits_response import AddBrandKitsResponse
from openapi_client.models.campaign.add_budgets_request import AddBudgetsRequest
from openapi_client.models.campaign.add_budgets_response import AddBudgetsResponse
from openapi_client.models.campaign.add_campaign_conversion_goals_request import AddCampaignConversionGoalsRequest
from openapi_client.models.campaign.add_campaign_conversion_goals_response import AddCampaignConversionGoalsResponse
from openapi_client.models.campaign.add_campaign_criterions_request import AddCampaignCriterionsRequest
from openapi_client.models.campaign.add_campaign_criterions_response import AddCampaignCriterionsResponse
from openapi_client.models.campaign.add_campaigns_request import AddCampaignsRequest
from openapi_client.models.campaign.add_campaigns_response import AddCampaignsResponse
from openapi_client.models.campaign.add_conversion_goals_request import AddConversionGoalsRequest
from openapi_client.models.campaign.add_conversion_goals_response import AddConversionGoalsResponse
from openapi_client.models.campaign.add_conversion_value_rules_request import AddConversionValueRulesRequest
from openapi_client.models.campaign.add_conversion_value_rules_response import AddConversionValueRulesResponse
from openapi_client.models.campaign.add_data_exclusions_request import AddDataExclusionsRequest
from openapi_client.models.campaign.add_data_exclusions_response import AddDataExclusionsResponse
from openapi_client.models.campaign.add_experiments_request import AddExperimentsRequest
from openapi_client.models.campaign.add_experiments_response import AddExperimentsResponse
from openapi_client.models.campaign.add_import_jobs_request import AddImportJobsRequest
from openapi_client.models.campaign.add_import_jobs_response import AddImportJobsResponse
from openapi_client.models.campaign.add_keywords_request import AddKeywordsRequest
from openapi_client.models.campaign.add_keywords_response import AddKeywordsResponse
from openapi_client.models.campaign.add_labels_request import AddLabelsRequest
from openapi_client.models.campaign.add_labels_response import AddLabelsResponse
from openapi_client.models.campaign.add_list_items_to_shared_list_request import AddListItemsToSharedListRequest
from openapi_client.models.campaign.add_list_items_to_shared_list_response import AddListItemsToSharedListResponse
from openapi_client.models.campaign.add_media_request import AddMediaRequest
from openapi_client.models.campaign.add_media_response import AddMediaResponse
from openapi_client.models.campaign.add_negative_keywords_to_entities_request import AddNegativeKeywordsToEntitiesRequest
from openapi_client.models.campaign.add_negative_keywords_to_entities_response import AddNegativeKeywordsToEntitiesResponse
from openapi_client.models.campaign.add_new_customer_acquisition_goals_request import AddNewCustomerAcquisitionGoalsRequest
from openapi_client.models.campaign.add_new_customer_acquisition_goals_response import AddNewCustomerAcquisitionGoalsResponse
from openapi_client.models.campaign.add_seasonality_adjustments_request import AddSeasonalityAdjustmentsRequest
from openapi_client.models.campaign.add_seasonality_adjustments_response import AddSeasonalityAdjustmentsResponse
from openapi_client.models.campaign.add_shared_entity_request import AddSharedEntityRequest
from openapi_client.models.campaign.add_shared_entity_response import AddSharedEntityResponse
from openapi_client.models.campaign.add_uet_tags_request import AddUetTagsRequest
from openapi_client.models.campaign.add_uet_tags_response import AddUetTagsResponse
from openapi_client.models.campaign.add_videos_request import AddVideosRequest
from openapi_client.models.campaign.add_videos_response import AddVideosResponse
from openapi_client.models.campaign.address import Address
from openapi_client.models.campaign.age_criterion import AgeCriterion
from openapi_client.models.campaign.age_dimension import AgeDimension
from openapi_client.models.campaign.age_range import AgeRange
from openapi_client.models.campaign.annotation_opt_out import AnnotationOptOut
from openapi_client.models.campaign.api_fault_detail import ApiFaultDetail
from openapi_client.models.campaign.app_ad_extension import AppAdExtension
from openapi_client.models.campaign.app_install_ad import AppInstallAd
from openapi_client.models.campaign.app_install_goal import AppInstallGoal
from openapi_client.models.campaign.app_setting import AppSetting
from openapi_client.models.campaign.app_store import AppStore
from openapi_client.models.campaign.app_url import AppUrl
from openapi_client.models.campaign.appeal_editorial_rejections_request import AppealEditorialRejectionsRequest
from openapi_client.models.campaign.appeal_editorial_rejections_response import AppealEditorialRejectionsResponse
from openapi_client.models.campaign.appeal_status import AppealStatus
from openapi_client.models.campaign.application_fault import ApplicationFault
from openapi_client.models.campaign.apply_asset_group_listing_group_actions_request import ApplyAssetGroupListingGroupActionsRequest
from openapi_client.models.campaign.apply_asset_group_listing_group_actions_response import ApplyAssetGroupListingGroupActionsResponse
from openapi_client.models.campaign.apply_customer_list_items_request import ApplyCustomerListItemsRequest
from openapi_client.models.campaign.apply_customer_list_items_response import ApplyCustomerListItemsResponse
from openapi_client.models.campaign.apply_customer_list_user_data_request import ApplyCustomerListUserDataRequest
from openapi_client.models.campaign.apply_customer_list_user_data_response import ApplyCustomerListUserDataResponse
from openapi_client.models.campaign.apply_hotel_group_actions_request import ApplyHotelGroupActionsRequest
from openapi_client.models.campaign.apply_hotel_group_actions_response import ApplyHotelGroupActionsResponse
from openapi_client.models.campaign.apply_offline_conversion_adjustments_request import ApplyOfflineConversionAdjustmentsRequest
from openapi_client.models.campaign.apply_offline_conversion_adjustments_response import ApplyOfflineConversionAdjustmentsResponse
from openapi_client.models.campaign.apply_offline_conversions_request import ApplyOfflineConversionsRequest
from openapi_client.models.campaign.apply_offline_conversions_response import ApplyOfflineConversionsResponse
from openapi_client.models.campaign.apply_online_conversion_adjustments_request import ApplyOnlineConversionAdjustmentsRequest
from openapi_client.models.campaign.apply_online_conversion_adjustments_response import ApplyOnlineConversionAdjustmentsResponse
from openapi_client.models.campaign.apply_product_partition_actions_request import ApplyProductPartitionActionsRequest
from openapi_client.models.campaign.apply_product_partition_actions_response import ApplyProductPartitionActionsResponse
from openapi_client.models.campaign.asset import Asset
from openapi_client.models.campaign.asset_group import AssetGroup
from openapi_client.models.campaign.asset_group_additional_field import AssetGroupAdditionalField
from openapi_client.models.campaign.asset_group_editorial_reason import AssetGroupEditorialReason
from openapi_client.models.campaign.asset_group_editorial_reason_collection import AssetGroupEditorialReasonCollection
from openapi_client.models.campaign.asset_group_editorial_status import AssetGroupEditorialStatus
from openapi_client.models.campaign.asset_group_listing_group import AssetGroupListingGroup
from openapi_client.models.campaign.asset_group_listing_group_action import AssetGroupListingGroupAction
from openapi_client.models.campaign.asset_group_listing_group_additional_field import AssetGroupListingGroupAdditionalField
from openapi_client.models.campaign.asset_group_listing_type import AssetGroupListingType
from openapi_client.models.campaign.asset_group_search_theme import AssetGroupSearchTheme
from openapi_client.models.campaign.asset_group_status import AssetGroupStatus
from openapi_client.models.campaign.asset_group_url_target import AssetGroupUrlTarget
from openapi_client.models.campaign.asset_link import AssetLink
from openapi_client.models.campaign.asset_link_editorial_status import AssetLinkEditorialStatus
from openapi_client.models.campaign.association_type import AssociationType
from openapi_client.models.campaign.attribution_model_type import AttributionModelType
from openapi_client.models.campaign.audience import Audience
from openapi_client.models.campaign.audience_additional_field import AudienceAdditionalField
from openapi_client.models.campaign.audience_condition import AudienceCondition
from openapi_client.models.campaign.audience_condition_item import AudienceConditionItem
from openapi_client.models.campaign.audience_criterion import AudienceCriterion
from openapi_client.models.campaign.audience_dimension import AudienceDimension
from openapi_client.models.campaign.audience_group import AudienceGroup
from openapi_client.models.campaign.audience_group_additional_field import AudienceGroupAdditionalField
from openapi_client.models.campaign.audience_group_asset_group_association import AudienceGroupAssetGroupAssociation
from openapi_client.models.campaign.audience_group_dimension import AudienceGroupDimension
from openapi_client.models.campaign.audience_group_dimension_type import AudienceGroupDimensionType
from openapi_client.models.campaign.audience_id_name import AudienceIdName
from openapi_client.models.campaign.audience_info import AudienceInfo
from openapi_client.models.campaign.audience_type import AudienceType
from openapi_client.models.campaign.audio_filter import AudioFilter
from openapi_client.models.campaign.authentication_fault_code import AuthenticationFaultCode
from openapi_client.models.campaign.bmc_store import BMCStore
from openapi_client.models.campaign.bmc_store_additional_field import BMCStoreAdditionalField
from openapi_client.models.campaign.bmc_store_sub_type import BMCStoreSubType
from openapi_client.models.campaign.batch_error import BatchError
from openapi_client.models.campaign.batch_error_base import BatchErrorBase
from openapi_client.models.campaign.batch_error_collection import BatchErrorCollection
from openapi_client.models.campaign.batch_error_collection_base import BatchErrorCollectionBase
from openapi_client.models.campaign.bid import Bid
from openapi_client.models.campaign.bid_multiplier import BidMultiplier
from openapi_client.models.campaign.bid_option import BidOption
from openapi_client.models.campaign.bid_strategy import BidStrategy
from openapi_client.models.campaign.biddable_ad_group_criterion import BiddableAdGroupCriterion
from openapi_client.models.campaign.biddable_campaign_criterion import BiddableCampaignCriterion
from openapi_client.models.campaign.bidding_scheme import BiddingScheme
from openapi_client.models.campaign.brand_item import BrandItem
from openapi_client.models.campaign.brand_kit import BrandKit
from openapi_client.models.campaign.brand_kit_additional_field import BrandKitAdditionalField
from openapi_client.models.campaign.brand_kit_color import BrandKitColor
from openapi_client.models.campaign.brand_kit_font import BrandKitFont
from openapi_client.models.campaign.brand_kit_image import BrandKitImage
from openapi_client.models.campaign.brand_kit_palette import BrandKitPalette
from openapi_client.models.campaign.brand_list import BrandList
from openapi_client.models.campaign.budget import Budget
from openapi_client.models.campaign.budget_limit_type import BudgetLimitType
from openapi_client.models.campaign.business_geo_code_status import BusinessGeoCodeStatus
from openapi_client.models.campaign.call_ad_extension import CallAdExtension
from openapi_client.models.campaign.call_to_action import CallToAction
from openapi_client.models.campaign.call_to_action_setting import CallToActionSetting
from openapi_client.models.campaign.callout_ad_extension import CalloutAdExtension
from openapi_client.models.campaign.campaign import Campaign
from openapi_client.models.campaign.campaign_ad_group_ids import CampaignAdGroupIds
from openapi_client.models.campaign.campaign_additional_field import CampaignAdditionalField
from openapi_client.models.campaign.campaign_association import CampaignAssociation
from openapi_client.models.campaign.campaign_conversion_goal import CampaignConversionGoal
from openapi_client.models.campaign.campaign_criterion import CampaignCriterion
from openapi_client.models.campaign.campaign_criterion_status import CampaignCriterionStatus
from openapi_client.models.campaign.campaign_criterion_type import CampaignCriterionType
from openapi_client.models.campaign.campaign_negative_sites import CampaignNegativeSites
from openapi_client.models.campaign.campaign_size import CampaignSize
from openapi_client.models.campaign.campaign_status import CampaignStatus
from openapi_client.models.campaign.campaign_type import CampaignType
from openapi_client.models.campaign.cashback_adjustment import CashbackAdjustment
from openapi_client.models.campaign.clipchamp_template_info import ClipchampTemplateInfo
from openapi_client.models.campaign.co_op_setting import CoOpSetting
from openapi_client.models.campaign.combination_rule import CombinationRule
from openapi_client.models.campaign.combined_list import CombinedList
from openapi_client.models.campaign.commission_bidding_scheme import CommissionBiddingScheme
from openapi_client.models.campaign.company import Company
from openapi_client.models.campaign.compression_type import CompressionType
from openapi_client.models.campaign.confirmation_action import ConfirmationAction
from openapi_client.models.campaign.conversion_goal import ConversionGoal
from openapi_client.models.campaign.conversion_goal_additional_field import ConversionGoalAdditionalField
from openapi_client.models.campaign.conversion_goal_category import ConversionGoalCategory
from openapi_client.models.campaign.conversion_goal_count_type import ConversionGoalCountType
from openapi_client.models.campaign.conversion_goal_revenue import ConversionGoalRevenue
from openapi_client.models.campaign.conversion_goal_revenue_type import ConversionGoalRevenueType
from openapi_client.models.campaign.conversion_goal_status import ConversionGoalStatus
from openapi_client.models.campaign.conversion_goal_tracking_status import ConversionGoalTrackingStatus
from openapi_client.models.campaign.conversion_goal_type import ConversionGoalType
from openapi_client.models.campaign.conversion_value_rule import ConversionValueRule
from openapi_client.models.campaign.conversion_value_rule_operator import ConversionValueRuleOperator
from openapi_client.models.campaign.conversion_value_rule_status import ConversionValueRuleStatus
from openapi_client.models.campaign.cost_per_sale_bidding_scheme import CostPerSaleBiddingScheme
from openapi_client.models.campaign.create_asset_group_recommendation_request import CreateAssetGroupRecommendationRequest
from openapi_client.models.campaign.create_asset_group_recommendation_response import CreateAssetGroupRecommendationResponse
from openapi_client.models.campaign.create_responsive_ad_recommendation_request import CreateResponsiveAdRecommendationRequest
from openapi_client.models.campaign.create_responsive_ad_recommendation_response import CreateResponsiveAdRecommendationResponse
from openapi_client.models.campaign.create_responsive_search_ad_recommendation_request import CreateResponsiveSearchAdRecommendationRequest
from openapi_client.models.campaign.create_responsive_search_ad_recommendation_response import CreateResponsiveSearchAdRecommendationResponse
from openapi_client.models.campaign.criterion import Criterion
from openapi_client.models.campaign.criterion_additional_field import CriterionAdditionalField
from openapi_client.models.campaign.criterion_bid import CriterionBid
from openapi_client.models.campaign.criterion_cashback import CriterionCashback
from openapi_client.models.campaign.criterion_type_group import CriterionTypeGroup
from openapi_client.models.campaign.custom_audience import CustomAudience
from openapi_client.models.campaign.custom_events_rule import CustomEventsRule
from openapi_client.models.campaign.custom_parameter import CustomParameter
from openapi_client.models.campaign.custom_parameters import CustomParameters
from openapi_client.models.campaign.customer_account_share import CustomerAccountShare
from openapi_client.models.campaign.customer_account_share_association import CustomerAccountShareAssociation
from openapi_client.models.campaign.customer_list import CustomerList
from openapi_client.models.campaign.customer_list_action_type import CustomerListActionType
from openapi_client.models.campaign.customer_list_item_sub_type import CustomerListItemSubType
from openapi_client.models.campaign.customer_list_user_data import CustomerListUserData
from openapi_client.models.campaign.customer_share import CustomerShare
from openapi_client.models.campaign.daily_summary import DailySummary
from openapi_client.models.campaign.data_exclusion import DataExclusion
from openapi_client.models.campaign.data_table_ad_extension import DataTableAdExtension
from openapi_client.models.campaign.day import Day
from openapi_client.models.campaign.day_time import DayTime
from openapi_client.models.campaign.day_time_criterion import DayTimeCriterion
from openapi_client.models.campaign.deal_criterion import DealCriterion
from openapi_client.models.campaign.delete_ad_extensions_associations_request import DeleteAdExtensionsAssociationsRequest
from openapi_client.models.campaign.delete_ad_extensions_associations_response import DeleteAdExtensionsAssociationsResponse
from openapi_client.models.campaign.delete_ad_extensions_request import DeleteAdExtensionsRequest
from openapi_client.models.campaign.delete_ad_extensions_response import DeleteAdExtensionsResponse
from openapi_client.models.campaign.delete_ad_group_criterions_request import DeleteAdGroupCriterionsRequest
from openapi_client.models.campaign.delete_ad_group_criterions_response import DeleteAdGroupCriterionsResponse
from openapi_client.models.campaign.delete_ad_groups_request import DeleteAdGroupsRequest
from openapi_client.models.campaign.delete_ad_groups_response import DeleteAdGroupsResponse
from openapi_client.models.campaign.delete_ads_request import DeleteAdsRequest
from openapi_client.models.campaign.delete_ads_response import DeleteAdsResponse
from openapi_client.models.campaign.delete_asset_groups_request import DeleteAssetGroupsRequest
from openapi_client.models.campaign.delete_asset_groups_response import DeleteAssetGroupsResponse
from openapi_client.models.campaign.delete_audience_group_asset_group_associations_request import DeleteAudienceGroupAssetGroupAssociationsRequest
from openapi_client.models.campaign.delete_audience_group_asset_group_associations_response import DeleteAudienceGroupAssetGroupAssociationsResponse
from openapi_client.models.campaign.delete_audience_groups_request import DeleteAudienceGroupsRequest
from openapi_client.models.campaign.delete_audience_groups_response import DeleteAudienceGroupsResponse
from openapi_client.models.campaign.delete_audiences_request import DeleteAudiencesRequest
from openapi_client.models.campaign.delete_audiences_response import DeleteAudiencesResponse
from openapi_client.models.campaign.delete_bid_strategies_request import DeleteBidStrategiesRequest
from openapi_client.models.campaign.delete_bid_strategies_response import DeleteBidStrategiesResponse
from openapi_client.models.campaign.delete_brand_kits_request import DeleteBrandKitsRequest
from openapi_client.models.campaign.delete_brand_kits_response import DeleteBrandKitsResponse
from openapi_client.models.campaign.delete_budgets_request import DeleteBudgetsRequest
from openapi_client.models.campaign.delete_budgets_response import DeleteBudgetsResponse
from openapi_client.models.campaign.delete_campaign_conversion_goals_request import DeleteCampaignConversionGoalsRequest
from openapi_client.models.campaign.delete_campaign_conversion_goals_response import DeleteCampaignConversionGoalsResponse
from openapi_client.models.campaign.delete_campaign_criterions_request import DeleteCampaignCriterionsRequest
from openapi_client.models.campaign.delete_campaign_criterions_response import DeleteCampaignCriterionsResponse
from openapi_client.models.campaign.delete_campaigns_request import DeleteCampaignsRequest
from openapi_client.models.campaign.delete_campaigns_response import DeleteCampaignsResponse
from openapi_client.models.campaign.delete_data_exclusions_request import DeleteDataExclusionsRequest
from openapi_client.models.campaign.delete_data_exclusions_response import DeleteDataExclusionsResponse
from openapi_client.models.campaign.delete_experiments_request import DeleteExperimentsRequest
from openapi_client.models.campaign.delete_experiments_response import DeleteExperimentsResponse
from openapi_client.models.campaign.delete_import_jobs_request import DeleteImportJobsRequest
from openapi_client.models.campaign.delete_import_jobs_response import DeleteImportJobsResponse
from openapi_client.models.campaign.delete_keywords_request import DeleteKeywordsRequest
from openapi_client.models.campaign.delete_keywords_response import DeleteKeywordsResponse
from openapi_client.models.campaign.delete_label_associations_request import DeleteLabelAssociationsRequest
from openapi_client.models.campaign.delete_label_associations_response import DeleteLabelAssociationsResponse
from openapi_client.models.campaign.delete_labels_request import DeleteLabelsRequest
from openapi_client.models.campaign.delete_labels_response import DeleteLabelsResponse
from openapi_client.models.campaign.delete_list_items_from_shared_list_request import DeleteListItemsFromSharedListRequest
from openapi_client.models.campaign.delete_list_items_from_shared_list_response import DeleteListItemsFromSharedListResponse
from openapi_client.models.campaign.delete_media_request import DeleteMediaRequest
from openapi_client.models.campaign.delete_media_response import DeleteMediaResponse
from openapi_client.models.campaign.delete_negative_keywords_from_entities_request import DeleteNegativeKeywordsFromEntitiesRequest
from openapi_client.models.campaign.delete_negative_keywords_from_entities_response import DeleteNegativeKeywordsFromEntitiesResponse
from openapi_client.models.campaign.delete_seasonality_adjustments_request import DeleteSeasonalityAdjustmentsRequest
from openapi_client.models.campaign.delete_seasonality_adjustments_response import DeleteSeasonalityAdjustmentsResponse
from openapi_client.models.campaign.delete_shared_entities_request import DeleteSharedEntitiesRequest
from openapi_client.models.campaign.delete_shared_entities_response import DeleteSharedEntitiesResponse
from openapi_client.models.campaign.delete_shared_entity_associations_request import DeleteSharedEntityAssociationsRequest
from openapi_client.models.campaign.delete_shared_entity_associations_response import DeleteSharedEntityAssociationsResponse
from openapi_client.models.campaign.delete_videos_request import DeleteVideosRequest
from openapi_client.models.campaign.delete_videos_response import DeleteVideosResponse
from openapi_client.models.campaign.device_condition import DeviceCondition
from openapi_client.models.campaign.device_criterion import DeviceCriterion
from openapi_client.models.campaign.device_type import DeviceType
from openapi_client.models.campaign.disclaimer_ad_extension import DisclaimerAdExtension
from openapi_client.models.campaign.disclaimer_setting import DisclaimerSetting
from openapi_client.models.campaign.distance_unit import DistanceUnit
from openapi_client.models.campaign.duration_goal import DurationGoal
from openapi_client.models.campaign.dynamic_feed_setting import DynamicFeedSetting
from openapi_client.models.campaign.dynamic_search_ad import DynamicSearchAd
from openapi_client.models.campaign.dynamic_search_ads_setting import DynamicSearchAdsSetting
from openapi_client.models.campaign.dynamic_search_ads_source import DynamicSearchAdsSource
from openapi_client.models.campaign.editorial_api_fault_detail import EditorialApiFaultDetail
from openapi_client.models.campaign.editorial_error import EditorialError
from openapi_client.models.campaign.editorial_error_collection import EditorialErrorCollection
from openapi_client.models.campaign.editorial_reason import EditorialReason
from openapi_client.models.campaign.editorial_reason_collection import EditorialReasonCollection
from openapi_client.models.campaign.enhanced_cpc_bidding_scheme import EnhancedCpcBiddingScheme
from openapi_client.models.campaign.entity_id_to_parent_id_association import EntityIdToParentIdAssociation
from openapi_client.models.campaign.entity_negative_keyword import EntityNegativeKeyword
from openapi_client.models.campaign.entity_scope import EntityScope
from openapi_client.models.campaign.entity_type import EntityType
from openapi_client.models.campaign.event_goal import EventGoal
from openapi_client.models.campaign.expanded_text_ad import ExpandedTextAd
from openapi_client.models.campaign.experiment import Experiment
from openapi_client.models.campaign.expression_operator import ExpressionOperator
from openapi_client.models.campaign.fault_source import FaultSource
from openapi_client.models.campaign.file_import_job import FileImportJob
from openapi_client.models.campaign.file_import_option import FileImportOption
from openapi_client.models.campaign.filter_link_ad_extension import FilterLinkAdExtension
from openapi_client.models.campaign.fixed_bid import FixedBid
from openapi_client.models.campaign.flyer_ad_extension import FlyerAdExtension
from openapi_client.models.campaign.font_text_asset_type import FontTextAssetType
from openapi_client.models.campaign.frequency import Frequency
from openapi_client.models.campaign.frequency_cap_settings import FrequencyCapSettings
from openapi_client.models.campaign.frequency_cap_time_granularity import FrequencyCapTimeGranularity
from openapi_client.models.campaign.gender_criterion import GenderCriterion
from openapi_client.models.campaign.gender_dimension import GenderDimension
from openapi_client.models.campaign.gender_type import GenderType
from openapi_client.models.campaign.generic_entity_status import GenericEntityStatus
from openapi_client.models.campaign.genre_criterion import GenreCriterion
from openapi_client.models.campaign.geo_point import GeoPoint
from openapi_client.models.campaign.get_account_migration_statuses_request import GetAccountMigrationStatusesRequest
from openapi_client.models.campaign.get_account_migration_statuses_response import GetAccountMigrationStatusesResponse
from openapi_client.models.campaign.get_account_properties_request import GetAccountPropertiesRequest
from openapi_client.models.campaign.get_account_properties_response import GetAccountPropertiesResponse
from openapi_client.models.campaign.get_ad_extension_ids_by_account_id_request import GetAdExtensionIdsByAccountIdRequest
from openapi_client.models.campaign.get_ad_extension_ids_by_account_id_response import GetAdExtensionIdsByAccountIdResponse
from openapi_client.models.campaign.get_ad_extensions_associations_request import GetAdExtensionsAssociationsRequest
from openapi_client.models.campaign.get_ad_extensions_associations_response import GetAdExtensionsAssociationsResponse
from openapi_client.models.campaign.get_ad_extensions_by_ids_request import GetAdExtensionsByIdsRequest
from openapi_client.models.campaign.get_ad_extensions_by_ids_response import GetAdExtensionsByIdsResponse
from openapi_client.models.campaign.get_ad_extensions_editorial_reasons_request import GetAdExtensionsEditorialReasonsRequest
from openapi_client.models.campaign.get_ad_extensions_editorial_reasons_response import GetAdExtensionsEditorialReasonsResponse
from openapi_client.models.campaign.get_ad_group_criterions_by_ids_request import GetAdGroupCriterionsByIdsRequest
from openapi_client.models.campaign.get_ad_group_criterions_by_ids_response import GetAdGroupCriterionsByIdsResponse
from openapi_client.models.campaign.get_ad_groups_by_campaign_id_request import GetAdGroupsByCampaignIdRequest
from openapi_client.models.campaign.get_ad_groups_by_campaign_id_response import GetAdGroupsByCampaignIdResponse
from openapi_client.models.campaign.get_ad_groups_by_ids_request import GetAdGroupsByIdsRequest
from openapi_client.models.campaign.get_ad_groups_by_ids_response import GetAdGroupsByIdsResponse
from openapi_client.models.campaign.get_ads_by_ad_group_id_request import GetAdsByAdGroupIdRequest
from openapi_client.models.campaign.get_ads_by_ad_group_id_response import GetAdsByAdGroupIdResponse
from openapi_client.models.campaign.get_ads_by_editorial_status_request import GetAdsByEditorialStatusRequest
from openapi_client.models.campaign.get_ads_by_editorial_status_response import GetAdsByEditorialStatusResponse
from openapi_client.models.campaign.get_ads_by_ids_request import GetAdsByIdsRequest
from openapi_client.models.campaign.get_ads_by_ids_response import GetAdsByIdsResponse
from openapi_client.models.campaign.get_annotation_opt_out_response import GetAnnotationOptOutResponse
from openapi_client.models.campaign.get_asset_group_listing_groups_by_ids_request import GetAssetGroupListingGroupsByIdsRequest
from openapi_client.models.campaign.get_asset_group_listing_groups_by_ids_response import GetAssetGroupListingGroupsByIdsResponse
from openapi_client.models.campaign.get_asset_groups_by_campaign_id_request import GetAssetGroupsByCampaignIdRequest
from openapi_client.models.campaign.get_asset_groups_by_campaign_id_response import GetAssetGroupsByCampaignIdResponse
from openapi_client.models.campaign.get_asset_groups_by_ids_request import GetAssetGroupsByIdsRequest
from openapi_client.models.campaign.get_asset_groups_by_ids_response import GetAssetGroupsByIdsResponse
from openapi_client.models.campaign.get_asset_groups_editorial_reasons_request import GetAssetGroupsEditorialReasonsRequest
from openapi_client.models.campaign.get_asset_groups_editorial_reasons_response import GetAssetGroupsEditorialReasonsResponse
from openapi_client.models.campaign.get_audience_group_asset_group_associations_by_asset_group_ids_request import GetAudienceGroupAssetGroupAssociationsByAssetGroupIdsRequest
from openapi_client.models.campaign.get_audience_group_asset_group_associations_by_asset_group_ids_response import GetAudienceGroupAssetGroupAssociationsByAssetGroupIdsResponse
from openapi_client.models.campaign.get_audience_group_asset_group_associations_by_audience_group_ids_request import GetAudienceGroupAssetGroupAssociationsByAudienceGroupIdsRequest
from openapi_client.models.campaign.get_audience_group_asset_group_associations_by_audience_group_ids_response import GetAudienceGroupAssetGroupAssociationsByAudienceGroupIdsResponse
from openapi_client.models.campaign.get_audience_groups_by_ids_request import GetAudienceGroupsByIdsRequest
from openapi_client.models.campaign.get_audience_groups_by_ids_response import GetAudienceGroupsByIdsResponse
from openapi_client.models.campaign.get_audiences_by_ids_request import GetAudiencesByIdsRequest
from openapi_client.models.campaign.get_audiences_by_ids_response import GetAudiencesByIdsResponse
from openapi_client.models.campaign.get_bmc_stores_by_customer_id_request import GetBMCStoresByCustomerIdRequest
from openapi_client.models.campaign.get_bmc_stores_by_customer_id_response import GetBMCStoresByCustomerIdResponse
from openapi_client.models.campaign.get_bsc_countries_response import GetBSCCountriesResponse
from openapi_client.models.campaign.get_bid_strategies_by_ids_request import GetBidStrategiesByIdsRequest
from openapi_client.models.campaign.get_bid_strategies_by_ids_response import GetBidStrategiesByIdsResponse
from openapi_client.models.campaign.get_brand_kits_by_account_id_request import GetBrandKitsByAccountIdRequest
from openapi_client.models.campaign.get_brand_kits_by_account_id_response import GetBrandKitsByAccountIdResponse
from openapi_client.models.campaign.get_brand_kits_by_ids_request import GetBrandKitsByIdsRequest
from openapi_client.models.campaign.get_brand_kits_by_ids_response import GetBrandKitsByIdsResponse
from openapi_client.models.campaign.get_budgets_by_ids_request import GetBudgetsByIdsRequest
from openapi_client.models.campaign.get_budgets_by_ids_response import GetBudgetsByIdsResponse
from openapi_client.models.campaign.get_campaign_criterions_by_ids_request import GetCampaignCriterionsByIdsRequest
from openapi_client.models.campaign.get_campaign_criterions_by_ids_response import GetCampaignCriterionsByIdsResponse
from openapi_client.models.campaign.get_campaign_ids_by_bid_strategy_ids_request import GetCampaignIdsByBidStrategyIdsRequest
from openapi_client.models.campaign.get_campaign_ids_by_bid_strategy_ids_response import GetCampaignIdsByBidStrategyIdsResponse
from openapi_client.models.campaign.get_campaign_ids_by_budget_ids_request import GetCampaignIdsByBudgetIdsRequest
from openapi_client.models.campaign.get_campaign_ids_by_budget_ids_response import GetCampaignIdsByBudgetIdsResponse
from openapi_client.models.campaign.get_campaign_sizes_by_account_id_request import GetCampaignSizesByAccountIdRequest
from openapi_client.models.campaign.get_campaign_sizes_by_account_id_response import GetCampaignSizesByAccountIdResponse
from openapi_client.models.campaign.get_campaigns_by_account_id_request import GetCampaignsByAccountIdRequest
from openapi_client.models.campaign.get_campaigns_by_account_id_response import GetCampaignsByAccountIdResponse
from openapi_client.models.campaign.get_campaigns_by_ids_request import GetCampaignsByIdsRequest
from openapi_client.models.campaign.get_campaigns_by_ids_response import GetCampaignsByIdsResponse
from openapi_client.models.campaign.get_clipchamp_templates_request import GetClipchampTemplatesRequest
from openapi_client.models.campaign.get_clipchamp_templates_response import GetClipchampTemplatesResponse
from openapi_client.models.campaign.get_config_value_request import GetConfigValueRequest
from openapi_client.models.campaign.get_config_value_response import GetConfigValueResponse
from openapi_client.models.campaign.get_conversion_goals_by_ids_request import GetConversionGoalsByIdsRequest
from openapi_client.models.campaign.get_conversion_goals_by_ids_response import GetConversionGoalsByIdsResponse
from openapi_client.models.campaign.get_conversion_goals_by_tag_ids_request import GetConversionGoalsByTagIdsRequest
from openapi_client.models.campaign.get_conversion_goals_by_tag_ids_response import GetConversionGoalsByTagIdsResponse
from openapi_client.models.campaign.get_conversion_value_rules_by_account_id_request import GetConversionValueRulesByAccountIdRequest
from openapi_client.models.campaign.get_conversion_value_rules_by_account_id_response import GetConversionValueRulesByAccountIdResponse
from openapi_client.models.campaign.get_conversion_value_rules_by_ids_request import GetConversionValueRulesByIdsRequest
from openapi_client.models.campaign.get_conversion_value_rules_by_ids_response import GetConversionValueRulesByIdsResponse
from openapi_client.models.campaign.get_data_exclusions_by_account_id_request import GetDataExclusionsByAccountIdRequest
from openapi_client.models.campaign.get_data_exclusions_by_account_id_response import GetDataExclusionsByAccountIdResponse
from openapi_client.models.campaign.get_data_exclusions_by_ids_request import GetDataExclusionsByIdsRequest
from openapi_client.models.campaign.get_data_exclusions_by_ids_response import GetDataExclusionsByIdsResponse
from openapi_client.models.campaign.get_editorial_reasons_by_ids_request import GetEditorialReasonsByIdsRequest
from openapi_client.models.campaign.get_editorial_reasons_by_ids_response import GetEditorialReasonsByIdsResponse
from openapi_client.models.campaign.get_experiments_by_ids_request import GetExperimentsByIdsRequest
from openapi_client.models.campaign.get_experiments_by_ids_response import GetExperimentsByIdsResponse
from openapi_client.models.campaign.get_file_import_upload_url_response import GetFileImportUploadUrlResponse
from openapi_client.models.campaign.get_geo_locations_file_url_request import GetGeoLocationsFileUrlRequest
from openapi_client.models.campaign.get_geo_locations_file_url_response import GetGeoLocationsFileUrlResponse
from openapi_client.models.campaign.get_health_check_request import GetHealthCheckRequest
from openapi_client.models.campaign.get_health_check_response import GetHealthCheckResponse
from openapi_client.models.campaign.get_import_entity_ids_mapping_request import GetImportEntityIdsMappingRequest
from openapi_client.models.campaign.get_import_entity_ids_mapping_response import GetImportEntityIdsMappingResponse
from openapi_client.models.campaign.get_import_jobs_by_ids_request import GetImportJobsByIdsRequest
from openapi_client.models.campaign.get_import_jobs_by_ids_response import GetImportJobsByIdsResponse
from openapi_client.models.campaign.get_import_results_request import GetImportResultsRequest
from openapi_client.models.campaign.get_import_results_response import GetImportResultsResponse
from openapi_client.models.campaign.get_keywords_by_ad_group_id_request import GetKeywordsByAdGroupIdRequest
from openapi_client.models.campaign.get_keywords_by_ad_group_id_response import GetKeywordsByAdGroupIdResponse
from openapi_client.models.campaign.get_keywords_by_editorial_status_request import GetKeywordsByEditorialStatusRequest
from openapi_client.models.campaign.get_keywords_by_editorial_status_response import GetKeywordsByEditorialStatusResponse
from openapi_client.models.campaign.get_keywords_by_ids_request import GetKeywordsByIdsRequest
from openapi_client.models.campaign.get_keywords_by_ids_response import GetKeywordsByIdsResponse
from openapi_client.models.campaign.get_label_associations_by_entity_ids_request import GetLabelAssociationsByEntityIdsRequest
from openapi_client.models.campaign.get_label_associations_by_entity_ids_response import GetLabelAssociationsByEntityIdsResponse
from openapi_client.models.campaign.get_label_associations_by_label_ids_request import GetLabelAssociationsByLabelIdsRequest
from openapi_client.models.campaign.get_label_associations_by_label_ids_response import GetLabelAssociationsByLabelIdsResponse
from openapi_client.models.campaign.get_labels_by_ids_request import GetLabelsByIdsRequest
from openapi_client.models.campaign.get_labels_by_ids_response import GetLabelsByIdsResponse
from openapi_client.models.campaign.get_list_items_by_shared_list_request import GetListItemsBySharedListRequest
from openapi_client.models.campaign.get_list_items_by_shared_list_response import GetListItemsBySharedListResponse
from openapi_client.models.campaign.get_media_associations_request import GetMediaAssociationsRequest
from openapi_client.models.campaign.get_media_associations_response import GetMediaAssociationsResponse
from openapi_client.models.campaign.get_media_meta_data_by_account_id_request import GetMediaMetaDataByAccountIdRequest
from openapi_client.models.campaign.get_media_meta_data_by_account_id_response import GetMediaMetaDataByAccountIdResponse
from openapi_client.models.campaign.get_media_meta_data_by_ids_request import GetMediaMetaDataByIdsRequest
from openapi_client.models.campaign.get_media_meta_data_by_ids_response import GetMediaMetaDataByIdsResponse
from openapi_client.models.campaign.get_negative_keywords_by_entity_ids_request import GetNegativeKeywordsByEntityIdsRequest
from openapi_client.models.campaign.get_negative_keywords_by_entity_ids_response import GetNegativeKeywordsByEntityIdsResponse
from openapi_client.models.campaign.get_negative_sites_by_ad_group_ids_request import GetNegativeSitesByAdGroupIdsRequest
from openapi_client.models.campaign.get_negative_sites_by_ad_group_ids_response import GetNegativeSitesByAdGroupIdsResponse
from openapi_client.models.campaign.get_negative_sites_by_campaign_ids_request import GetNegativeSitesByCampaignIdsRequest
from openapi_client.models.campaign.get_negative_sites_by_campaign_ids_response import GetNegativeSitesByCampaignIdsResponse
from openapi_client.models.campaign.get_new_customer_acquisition_goals_by_account_id_request import GetNewCustomerAcquisitionGoalsByAccountIdRequest
from openapi_client.models.campaign.get_new_customer_acquisition_goals_by_account_id_response import GetNewCustomerAcquisitionGoalsByAccountIdResponse
from openapi_client.models.campaign.get_offline_conversion_reports_request import GetOfflineConversionReportsRequest
from openapi_client.models.campaign.get_offline_conversion_reports_response import GetOfflineConversionReportsResponse
from openapi_client.models.campaign.get_profile_data_file_url_request import GetProfileDataFileUrlRequest
from openapi_client.models.campaign.get_profile_data_file_url_response import GetProfileDataFileUrlResponse
from openapi_client.models.campaign.get_seasonality_adjustments_by_account_id_request import GetSeasonalityAdjustmentsByAccountIdRequest
from openapi_client.models.campaign.get_seasonality_adjustments_by_account_id_response import GetSeasonalityAdjustmentsByAccountIdResponse
from openapi_client.models.campaign.get_seasonality_adjustments_by_ids_request import GetSeasonalityAdjustmentsByIdsRequest
from openapi_client.models.campaign.get_seasonality_adjustments_by_ids_response import GetSeasonalityAdjustmentsByIdsResponse
from openapi_client.models.campaign.get_shared_entities_by_account_id_request import GetSharedEntitiesByAccountIdRequest
from openapi_client.models.campaign.get_shared_entities_by_account_id_response import GetSharedEntitiesByAccountIdResponse
from openapi_client.models.campaign.get_shared_entities_request import GetSharedEntitiesRequest
from openapi_client.models.campaign.get_shared_entities_response import GetSharedEntitiesResponse
from openapi_client.models.campaign.get_shared_entity_associations_by_entity_ids_request import GetSharedEntityAssociationsByEntityIdsRequest
from openapi_client.models.campaign.get_shared_entity_associations_by_entity_ids_response import GetSharedEntityAssociationsByEntityIdsResponse
from openapi_client.models.campaign.get_shared_entity_associations_by_shared_entity_ids_request import GetSharedEntityAssociationsBySharedEntityIdsRequest
from openapi_client.models.campaign.get_shared_entity_associations_by_shared_entity_ids_response import GetSharedEntityAssociationsBySharedEntityIdsResponse
from openapi_client.models.campaign.get_supported_clipchamp_audio_request import GetSupportedClipchampAudioRequest
from openapi_client.models.campaign.get_supported_clipchamp_audio_response import GetSupportedClipchampAudioResponse
from openapi_client.models.campaign.get_supported_fonts_request import GetSupportedFontsRequest
from openapi_client.models.campaign.get_supported_fonts_response import GetSupportedFontsResponse
from openapi_client.models.campaign.get_uet_tags_by_ids_request import GetUetTagsByIdsRequest
from openapi_client.models.campaign.get_uet_tags_by_ids_response import GetUetTagsByIdsResponse
from openapi_client.models.campaign.get_videos_by_ids_request import GetVideosByIdsRequest
from openapi_client.models.campaign.get_videos_by_ids_response import GetVideosByIdsResponse
from openapi_client.models.campaign.google_import_job import GoogleImportJob
from openapi_client.models.campaign.google_import_option import GoogleImportOption
from openapi_client.models.campaign.health_check_action_link_metadata import HealthCheckActionLinkMetadata
from openapi_client.models.campaign.health_check_column_metadata import HealthCheckColumnMetadata
from openapi_client.models.campaign.health_check_data import HealthCheckData
from openapi_client.models.campaign.health_check_entity import HealthCheckEntity
from openapi_client.models.campaign.health_check_error import HealthCheckError
from openapi_client.models.campaign.health_check_metadata import HealthCheckMetadata
from openapi_client.models.campaign.health_check_sub_entity_data import HealthCheckSubEntityData
from openapi_client.models.campaign.hotel_ad import HotelAd
from openapi_client.models.campaign.hotel_ad_group_type import HotelAdGroupType
from openapi_client.models.campaign.hotel_advance_booking_window_criterion import HotelAdvanceBookingWindowCriterion
from openapi_client.models.campaign.hotel_check_in_date_criterion import HotelCheckInDateCriterion
from openapi_client.models.campaign.hotel_check_in_day_criterion import HotelCheckInDayCriterion
from openapi_client.models.campaign.hotel_date_selection_type import HotelDateSelectionType
from openapi_client.models.campaign.hotel_date_selection_type_criterion import HotelDateSelectionTypeCriterion
from openapi_client.models.campaign.hotel_group import HotelGroup
from openapi_client.models.campaign.hotel_length_of_stay_criterion import HotelLengthOfStayCriterion
from openapi_client.models.campaign.hotel_listing import HotelListing
from openapi_client.models.campaign.hotel_listing_type import HotelListingType
from openapi_client.models.campaign.hotel_setting import HotelSetting
from openapi_client.models.campaign.id_collection import IdCollection
from openapi_client.models.campaign.image import Image
from openapi_client.models.campaign.image_ad_extension import ImageAdExtension
from openapi_client.models.campaign.image_asset import ImageAsset
from openapi_client.models.campaign.image_media_representation import ImageMediaRepresentation
from openapi_client.models.campaign.import_additional_field import ImportAdditionalField
from openapi_client.models.campaign.import_entity_statistics import ImportEntityStatistics
from openapi_client.models.campaign.import_entity_type import ImportEntityType
from openapi_client.models.campaign.import_job import ImportJob
from openapi_client.models.campaign.import_option import ImportOption
from openapi_client.models.campaign.import_result import ImportResult
from openapi_client.models.campaign.import_search_and_replace_for_string_property import ImportSearchAndReplaceForStringProperty
from openapi_client.models.campaign.impression_based_entity_type import ImpressionBasedEntityType
from openapi_client.models.campaign.impression_based_remarketing_list import ImpressionBasedRemarketingList
from openapi_client.models.campaign.in_market_audience import InMarketAudience
from openapi_client.models.campaign.in_store_transaction_goal import InStoreTransactionGoal
from openapi_client.models.campaign.inherit_from_parent_bidding_scheme import InheritFromParentBiddingScheme
from openapi_client.models.campaign.intent_option import IntentOption
from openapi_client.models.campaign.item_action import ItemAction
from openapi_client.models.campaign.key_frame import KeyFrame
from openapi_client.models.campaign.key_value_pair_oflong_andlong import KeyValuePairOflongAndlong
from openapi_client.models.campaign.key_value_pair_ofstring_andstring import KeyValuePairOfstringAndstring
from openapi_client.models.campaign.keyword import Keyword
from openapi_client.models.campaign.keyword_editorial_status import KeywordEditorialStatus
from openapi_client.models.campaign.keyword_status import KeywordStatus
from openapi_client.models.campaign.label import Label
from openapi_client.models.campaign.label_association import LabelAssociation
from openapi_client.models.campaign.language_name import LanguageName
from openapi_client.models.campaign.lead_delivery import LeadDelivery
from openapi_client.models.campaign.lead_form_ad_extension import LeadFormAdExtension
from openapi_client.models.campaign.lead_form_call_to_action import LeadFormCallToAction
from openapi_client.models.campaign.lead_form_question_definition import LeadFormQuestionDefinition
from openapi_client.models.campaign.location_ad_extension import LocationAdExtension
from openapi_client.models.campaign.location_condition import LocationCondition
from openapi_client.models.campaign.location_condition_item import LocationConditionItem
from openapi_client.models.campaign.location_criterion import LocationCriterion
from openapi_client.models.campaign.location_intent_criterion import LocationIntentCriterion
from openapi_client.models.campaign.location_type import LocationType
from openapi_client.models.campaign.logical_operator import LogicalOperator
from openapi_client.models.campaign.logo_ad_extension import LogoAdExtension
from openapi_client.models.campaign.manual_cpa_bidding_scheme import ManualCpaBiddingScheme
from openapi_client.models.campaign.manual_cpc_bidding_scheme import ManualCpcBiddingScheme
from openapi_client.models.campaign.manual_cpm_bidding_scheme import ManualCpmBiddingScheme
from openapi_client.models.campaign.manual_cpv_bidding_scheme import ManualCpvBiddingScheme
from openapi_client.models.campaign.match_type import MatchType
from openapi_client.models.campaign.max_clicks_bidding_scheme import MaxClicksBiddingScheme
from openapi_client.models.campaign.max_conversion_value_bidding_scheme import MaxConversionValueBiddingScheme
from openapi_client.models.campaign.max_conversions_bidding_scheme import MaxConversionsBiddingScheme
from openapi_client.models.campaign.max_roas_bidding_scheme import MaxRoasBiddingScheme
from openapi_client.models.campaign.media import Media
from openapi_client.models.campaign.media_additional_field import MediaAdditionalField
from openapi_client.models.campaign.media_association import MediaAssociation
from openapi_client.models.campaign.media_enabled_entity_filter import MediaEnabledEntityFilter
from openapi_client.models.campaign.media_meta_data import MediaMetaData
from openapi_client.models.campaign.media_representation import MediaRepresentation
from openapi_client.models.campaign.media_representation_base import MediaRepresentationBase
from openapi_client.models.campaign.migration_status import MigrationStatus
from openapi_client.models.campaign.migration_status_info import MigrationStatusInfo
from openapi_client.models.campaign.minute import Minute
from openapi_client.models.campaign.model_date import ModelDate
from openapi_client.models.campaign.negative_ad_group_criterion import NegativeAdGroupCriterion
from openapi_client.models.campaign.negative_campaign_criterion import NegativeCampaignCriterion
from openapi_client.models.campaign.negative_keyword import NegativeKeyword
from openapi_client.models.campaign.negative_keyword_list import NegativeKeywordList
from openapi_client.models.campaign.negative_site import NegativeSite
from openapi_client.models.campaign.network import Network
from openapi_client.models.campaign.new_customer_acquisition_goal import NewCustomerAcquisitionGoal
from openapi_client.models.campaign.new_customer_acquisition_goal_setting import NewCustomerAcquisitionGoalSetting
from openapi_client.models.campaign.news_ad_extension import NewsAdExtension
from openapi_client.models.campaign.normal_form import NormalForm
from openapi_client.models.campaign.number_operator import NumberOperator
from openapi_client.models.campaign.number_rule_item import NumberRuleItem
from openapi_client.models.campaign.offline_conversion import OfflineConversion
from openapi_client.models.campaign.offline_conversion_adjustment import OfflineConversionAdjustment
from openapi_client.models.campaign.offline_conversion_goal import OfflineConversionGoal
from openapi_client.models.campaign.online_conversion_adjustment import OnlineConversionAdjustment
from openapi_client.models.campaign.operation_error import OperationError
from openapi_client.models.campaign.page_visitors_rule import PageVisitorsRule
from openapi_client.models.campaign.page_visitors_who_did_not_visit_another_page_rule import PageVisitorsWhoDidNotVisitAnotherPageRule
from openapi_client.models.campaign.page_visitors_who_visited_another_page_rule import PageVisitorsWhoVisitedAnotherPageRule
from openapi_client.models.campaign.pages_viewed_per_visit_goal import PagesViewedPerVisitGoal
from openapi_client.models.campaign.paging import Paging
from openapi_client.models.campaign.percent_cpc_bidding_scheme import PercentCpcBiddingScheme
from openapi_client.models.campaign.performance_max_setting import PerformanceMaxSetting
from openapi_client.models.campaign.placement_criterion import PlacementCriterion
from openapi_client.models.campaign.placement_exclusion_list import PlacementExclusionList
from openapi_client.models.campaign.price_ad_extension import PriceAdExtension
from openapi_client.models.campaign.price_extension_type import PriceExtensionType
from openapi_client.models.campaign.price_qualifier import PriceQualifier
from openapi_client.models.campaign.price_table_row import PriceTableRow
from openapi_client.models.campaign.price_unit import PriceUnit
from openapi_client.models.campaign.product_ad import ProductAd
from openapi_client.models.campaign.product_audience import ProductAudience
from openapi_client.models.campaign.product_audience_type import ProductAudienceType
from openapi_client.models.campaign.product_condition import ProductCondition
from openapi_client.models.campaign.product_partition import ProductPartition
from openapi_client.models.campaign.product_partition_type import ProductPartitionType
from openapi_client.models.campaign.product_scope import ProductScope
from openapi_client.models.campaign.profile_criterion import ProfileCriterion
from openapi_client.models.campaign.profile_dimension import ProfileDimension
from openapi_client.models.campaign.profile_info import ProfileInfo
from openapi_client.models.campaign.profile_type import ProfileType
from openapi_client.models.campaign.promotion_ad_extension import PromotionAdExtension
from openapi_client.models.campaign.promotion_discount_modifier import PromotionDiscountModifier
from openapi_client.models.campaign.promotion_occasion import PromotionOccasion
from openapi_client.models.campaign.radius_criterion import RadiusCriterion
from openapi_client.models.campaign.rate_amount import RateAmount
from openapi_client.models.campaign.rate_bid import RateBid
from openapi_client.models.campaign.recommendation_asset_property import RecommendationAssetProperty
from openapi_client.models.campaign.recommendation_audio import RecommendationAudio
from openapi_client.models.campaign.recommendation_background import RecommendationBackground
from openapi_client.models.campaign.recommendation_business_name import RecommendationBusinessName
from openapi_client.models.campaign.recommendation_color import RecommendationColor
from openapi_client.models.campaign.recommendation_crop_setting import RecommendationCropSetting
from openapi_client.models.campaign.recommendation_image import RecommendationImage
from openapi_client.models.campaign.recommendation_image_size import RecommendationImageSize
from openapi_client.models.campaign.recommendation_logo import RecommendationLogo
from openapi_client.models.campaign.recommendation_scene import RecommendationScene
from openapi_client.models.campaign.recommendation_template import RecommendationTemplate
from openapi_client.models.campaign.recommendation_template_video import RecommendationTemplateVideo
from openapi_client.models.campaign.recommendation_text import RecommendationText
from openapi_client.models.campaign.refine_asset_group_recommendation_request import RefineAssetGroupRecommendationRequest
from openapi_client.models.campaign.refine_asset_group_recommendation_response import RefineAssetGroupRecommendationResponse
from openapi_client.models.campaign.refine_responsive_ad_recommendation_request import RefineResponsiveAdRecommendationRequest
from openapi_client.models.campaign.refine_responsive_ad_recommendation_response import RefineResponsiveAdRecommendationResponse
from openapi_client.models.campaign.refine_responsive_search_ad_recommendation_request import RefineResponsiveSearchAdRecommendationRequest
from openapi_client.models.campaign.refine_responsive_search_ad_recommendation_response import RefineResponsiveSearchAdRecommendationResponse
from openapi_client.models.campaign.remarketing_list import RemarketingList
from openapi_client.models.campaign.remarketing_rule import RemarketingRule
from openapi_client.models.campaign.responsive_ad import ResponsiveAd
from openapi_client.models.campaign.responsive_search_ad import ResponsiveSearchAd
from openapi_client.models.campaign.responsive_search_ads_setting import ResponsiveSearchAdsSetting
from openapi_client.models.campaign.review_ad_extension import ReviewAdExtension
from openapi_client.models.campaign.rule_item import RuleItem
from openapi_client.models.campaign.rule_item_group import RuleItemGroup
from openapi_client.models.campaign.schedule import Schedule
from openapi_client.models.campaign.search_companies_request import SearchCompaniesRequest
from openapi_client.models.campaign.search_companies_response import SearchCompaniesResponse
from openapi_client.models.campaign.seasonality_adjustment import SeasonalityAdjustment
from openapi_client.models.campaign.set_account_properties_request import SetAccountPropertiesRequest
from openapi_client.models.campaign.set_ad_extensions_associations_request import SetAdExtensionsAssociationsRequest
from openapi_client.models.campaign.set_ad_extensions_associations_response import SetAdExtensionsAssociationsResponse
from openapi_client.models.campaign.set_audience_group_asset_group_associations_request import SetAudienceGroupAssetGroupAssociationsRequest
from openapi_client.models.campaign.set_audience_group_asset_group_associations_response import SetAudienceGroupAssetGroupAssociationsResponse
from openapi_client.models.campaign.set_label_associations_request import SetLabelAssociationsRequest
from openapi_client.models.campaign.set_label_associations_response import SetLabelAssociationsResponse
from openapi_client.models.campaign.set_negative_sites_to_ad_groups_request import SetNegativeSitesToAdGroupsRequest
from openapi_client.models.campaign.set_negative_sites_to_ad_groups_response import SetNegativeSitesToAdGroupsResponse
from openapi_client.models.campaign.set_negative_sites_to_campaigns_request import SetNegativeSitesToCampaignsRequest
from openapi_client.models.campaign.set_negative_sites_to_campaigns_response import SetNegativeSitesToCampaignsResponse
from openapi_client.models.campaign.set_shared_entity_associations_request import SetSharedEntityAssociationsRequest
from openapi_client.models.campaign.set_shared_entity_associations_response import SetSharedEntityAssociationsResponse
from openapi_client.models.campaign.setting import Setting
from openapi_client.models.campaign.shared_entity import SharedEntity
from openapi_client.models.campaign.shared_entity_association import SharedEntityAssociation
from openapi_client.models.campaign.shared_list import SharedList
from openapi_client.models.campaign.shared_list_item import SharedListItem
from openapi_client.models.campaign.shopping_setting import ShoppingSetting
from openapi_client.models.campaign.similar_remarketing_list import SimilarRemarketingList
from openapi_client.models.campaign.site import Site
from openapi_client.models.campaign.sitelink_ad_extension import SitelinkAdExtension
from openapi_client.models.campaign.solid_background_asset import SolidBackgroundAsset
from openapi_client.models.campaign.store_criterion import StoreCriterion
from openapi_client.models.campaign.string_operator import StringOperator
from openapi_client.models.campaign.string_rule_item import StringRuleItem
from openapi_client.models.campaign.structured_snippet_ad_extension import StructuredSnippetAdExtension
from openapi_client.models.campaign.supported_clipchamp_audio import SupportedClipchampAudio
from openapi_client.models.campaign.supported_clipchamp_audio_data import SupportedClipchampAudioData
from openapi_client.models.campaign.supported_font import SupportedFont
from openapi_client.models.campaign.supported_fonts_data import SupportedFontsData
from openapi_client.models.campaign.target_cpa_bidding_scheme import TargetCpaBiddingScheme
from openapi_client.models.campaign.target_impression_share_bidding_scheme import TargetImpressionShareBiddingScheme
from openapi_client.models.campaign.target_roas_bidding_scheme import TargetRoasBiddingScheme
from openapi_client.models.campaign.target_setting import TargetSetting
from openapi_client.models.campaign.target_setting_detail import TargetSettingDetail
from openapi_client.models.campaign.text_ad import TextAd
from openapi_client.models.campaign.text_asset import TextAsset
from openapi_client.models.campaign.third_party_measurement_setting import ThirdPartyMeasurementSetting
from openapi_client.models.campaign.uet_tag import UetTag
from openapi_client.models.campaign.uet_tag_tracking_status import UetTagTrackingStatus
from openapi_client.models.campaign.update_ad_extensions_request import UpdateAdExtensionsRequest
from openapi_client.models.campaign.update_ad_extensions_response import UpdateAdExtensionsResponse
from openapi_client.models.campaign.update_ad_group_criterions_request import UpdateAdGroupCriterionsRequest
from openapi_client.models.campaign.update_ad_group_criterions_response import UpdateAdGroupCriterionsResponse
from openapi_client.models.campaign.update_ad_groups_request import UpdateAdGroupsRequest
from openapi_client.models.campaign.update_ad_groups_response import UpdateAdGroupsResponse
from openapi_client.models.campaign.update_ads_request import UpdateAdsRequest
from openapi_client.models.campaign.update_ads_response import UpdateAdsResponse
from openapi_client.models.campaign.update_annotation_opt_out_request import UpdateAnnotationOptOutRequest
from openapi_client.models.campaign.update_asset_groups_request import UpdateAssetGroupsRequest
from openapi_client.models.campaign.update_asset_groups_response import UpdateAssetGroupsResponse
from openapi_client.models.campaign.update_audience_groups_request import UpdateAudienceGroupsRequest
from openapi_client.models.campaign.update_audience_groups_response import UpdateAudienceGroupsResponse
from openapi_client.models.campaign.update_audiences_request import UpdateAudiencesRequest
from openapi_client.models.campaign.update_audiences_response import UpdateAudiencesResponse
from openapi_client.models.campaign.update_bid_strategies_request import UpdateBidStrategiesRequest
from openapi_client.models.campaign.update_bid_strategies_response import UpdateBidStrategiesResponse
from openapi_client.models.campaign.update_brand_kits_request import UpdateBrandKitsRequest
from openapi_client.models.campaign.update_brand_kits_response import UpdateBrandKitsResponse
from openapi_client.models.campaign.update_budgets_request import UpdateBudgetsRequest
from openapi_client.models.campaign.update_budgets_response import UpdateBudgetsResponse
from openapi_client.models.campaign.update_campaign_criterions_request import UpdateCampaignCriterionsRequest
from openapi_client.models.campaign.update_campaign_criterions_response import UpdateCampaignCriterionsResponse
from openapi_client.models.campaign.update_campaigns_request import UpdateCampaignsRequest
from openapi_client.models.campaign.update_campaigns_response import UpdateCampaignsResponse
from openapi_client.models.campaign.update_conversion_goals_request import UpdateConversionGoalsRequest
from openapi_client.models.campaign.update_conversion_goals_response import UpdateConversionGoalsResponse
from openapi_client.models.campaign.update_conversion_value_rules_request import UpdateConversionValueRulesRequest
from openapi_client.models.campaign.update_conversion_value_rules_response import UpdateConversionValueRulesResponse
from openapi_client.models.campaign.update_conversion_value_rules_status_request import UpdateConversionValueRulesStatusRequest
from openapi_client.models.campaign.update_conversion_value_rules_status_response import UpdateConversionValueRulesStatusResponse
from openapi_client.models.campaign.update_data_exclusions_request import UpdateDataExclusionsRequest
from openapi_client.models.campaign.update_data_exclusions_response import UpdateDataExclusionsResponse
from openapi_client.models.campaign.update_experiments_request import UpdateExperimentsRequest
from openapi_client.models.campaign.update_experiments_response import UpdateExperimentsResponse
from openapi_client.models.campaign.update_import_jobs_request import UpdateImportJobsRequest
from openapi_client.models.campaign.update_import_jobs_response import UpdateImportJobsResponse
from openapi_client.models.campaign.update_keywords_request import UpdateKeywordsRequest
from openapi_client.models.campaign.update_keywords_response import UpdateKeywordsResponse
from openapi_client.models.campaign.update_labels_request import UpdateLabelsRequest
from openapi_client.models.campaign.update_labels_response import UpdateLabelsResponse
from openapi_client.models.campaign.update_new_customer_acquisition_goals_request import UpdateNewCustomerAcquisitionGoalsRequest
from openapi_client.models.campaign.update_new_customer_acquisition_goals_response import UpdateNewCustomerAcquisitionGoalsResponse
from openapi_client.models.campaign.update_seasonality_adjustments_request import UpdateSeasonalityAdjustmentsRequest
from openapi_client.models.campaign.update_seasonality_adjustments_response import UpdateSeasonalityAdjustmentsResponse
from openapi_client.models.campaign.update_shared_entities_request import UpdateSharedEntitiesRequest
from openapi_client.models.campaign.update_shared_entities_response import UpdateSharedEntitiesResponse
from openapi_client.models.campaign.update_uet_tags_request import UpdateUetTagsRequest
from openapi_client.models.campaign.update_uet_tags_response import UpdateUetTagsResponse
from openapi_client.models.campaign.update_videos_request import UpdateVideosRequest
from openapi_client.models.campaign.update_videos_response import UpdateVideosResponse
from openapi_client.models.campaign.url_goal import UrlGoal
from openapi_client.models.campaign.value_operator import ValueOperator
from openapi_client.models.campaign.vanity_pharma_display_url_mode import VanityPharmaDisplayUrlMode
from openapi_client.models.campaign.vanity_pharma_setting import VanityPharmaSetting
from openapi_client.models.campaign.vanity_pharma_website_description import VanityPharmaWebsiteDescription
from openapi_client.models.campaign.verified_tracking_setting import VerifiedTrackingSetting
from openapi_client.models.campaign.video import Video
from openapi_client.models.campaign.video_ad_extension import VideoAdExtension
from openapi_client.models.campaign.video_asset import VideoAsset
from openapi_client.models.campaign.video_recommendation_image_data import VideoRecommendationImageData
from openapi_client.models.campaign.video_recommendation_text_data import VideoRecommendationTextData
from openapi_client.models.campaign.video_template_filter import VideoTemplateFilter
from openapi_client.models.campaign.webpage import Webpage
from openapi_client.models.campaign.webpage_condition import WebpageCondition
from openapi_client.models.campaign.webpage_condition_operand import WebpageConditionOperand
from openapi_client.models.campaign.webpage_condition_operator import WebpageConditionOperator
from openapi_client.models.campaign.webpage_parameter import WebpageParameter



from bingads.v13.bulk.entities import *
from bingads.v13.bulk.entities.bulk_ad_group_hotel_listing_group import BulkAdGroupHotelListingGroup
from bingads.v13.bulk.entities.bulk_online_conversion_adjustment import BulkOnlineConversionAdjustment
from bingads.v13.bulk.entities.target_criterions.bulk_ad_group_hotel_advance_booking_window_criterion import \
    BulkAdGroupHotelAdvanceBookingWindowCriterion
from bingads.v13.bulk.entities.target_criterions.bulk_ad_group_hotel_check_in_date_criterion import \
    BulkAdGroupHotelCheckInDateCriterion
from bingads.v13.bulk.entities.target_criterions.bulk_ad_group_hotel_check_in_day_criterion import \
    BulkAdGroupHotelCheckInDayCriterion
from bingads.v13.bulk.entities.target_criterions.bulk_ad_group_hotel_date_selection_type_criterion import \
    BulkAdGroupHotelDateSelectionTypeCriterion
from bingads.v13.bulk.entities.target_criterions.bulk_ad_group_hotel_length_of_stay_criterion import \
    BulkAdGroupHotelLengthOfStayCriterion
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.bulk.entity_info import _EntityInfo
from bingads.v13.bulk.entities.bulk_negative_sites import _BulkAdGroupNegativeSitesIdentifier, \
    _BulkCampaignNegativeSitesIdentifier
from bingads.v13.internal.bulk.format_version import _FormatVersion


class _BulkObjectFactory():
    INDIVIDUAL_ENTITY_MAP = {
        _StringTable.Image: _EntityInfo(lambda: BulkImage()),
        _StringTable.Video: _EntityInfo(lambda: BulkVideo()),
        _StringTable.Account: _EntityInfo(lambda: BulkAccount()),
        _StringTable.Budget: _EntityInfo(lambda: BulkBudget()),
        _StringTable.BidStrategy: _EntityInfo(lambda: BulkBidStrategy()),
        _StringTable.Campaign: _EntityInfo(lambda: BulkCampaign()),
        _StringTable.AdGroup: _EntityInfo(lambda: BulkAdGroup()),
        _StringTable.Keyword: _EntityInfo(lambda: BulkKeyword()),
        _StringTable.VideoAdExtension: _EntityInfo(lambda: BulkVideoAdExtension()),
        _StringTable.AccountVideoAdExtension: _EntityInfo(lambda: BulkAccountVideoAdExtension()),
        _StringTable.CampaignVideoAdExtension: _EntityInfo(lambda: BulkCampaignVideoAdExtension()),
        _StringTable.AdGroupVideoAdExtension: _EntityInfo(lambda: BulkAdGroupVideoAdExtension()),
        _StringTable.CallAdExtension: _EntityInfo(lambda: BulkCallAdExtension()),
        _StringTable.CampaignCallAdExtension: _EntityInfo(lambda: BulkCampaignCallAdExtension()),
        _StringTable.FlyerAdExtension: _EntityInfo(lambda: BulkFlyerAdExtension()),
        _StringTable.AccountFlyerAdExtension: _EntityInfo(lambda: BulkAccountFlyerAdExtension()),
        _StringTable.CampaignFlyerAdExtension: _EntityInfo(lambda: BulkCampaignFlyerAdExtension()),
        _StringTable.AdGroupFlyerAdExtension: _EntityInfo(lambda: BulkAdGroupFlyerAdExtension()),
        _StringTable.DisclaimerAdExtension: _EntityInfo(lambda: BulkDisclaimerAdExtension()),
        _StringTable.CampaignDisclaimerAdExtension: _EntityInfo(lambda: BulkCampaignDisclaimerAdExtension()),
        _StringTable.ImageAdExtension: _EntityInfo(lambda: BulkImageAdExtension()),
        _StringTable.AccountImageAdExtension: _EntityInfo(lambda: BulkAccountImageAdExtension()),
        _StringTable.CampaignImageAdExtension: _EntityInfo(lambda: BulkCampaignImageAdExtension()),
        _StringTable.AdGroupImageAdExtension: _EntityInfo(lambda: BulkAdGroupImageAdExtension()),
        _StringTable.FilterLinkAdExtension: _EntityInfo(lambda: BulkFilterLinkAdExtension()),
        _StringTable.AccountFilterLinkAdExtension: _EntityInfo(lambda: BulkAccountFilterLinkAdExtension()),
        _StringTable.CampaignFilterLinkAdExtension: _EntityInfo(lambda: BulkCampaignFilterLinkAdExtension()),
        _StringTable.AdGroupFilterLinkAdExtension: _EntityInfo(lambda: BulkAdGroupFilterLinkAdExtension()),
        _StringTable.CalloutAdExtension: _EntityInfo(lambda: BulkCalloutAdExtension()),
        _StringTable.AccountCalloutAdExtension: _EntityInfo(lambda: BulkAccountCalloutAdExtension()),
        _StringTable.CampaignCalloutAdExtension: _EntityInfo(lambda: BulkCampaignCalloutAdExtension()),
        _StringTable.AdGroupCalloutAdExtension: _EntityInfo(lambda: BulkAdGroupCalloutAdExtension()),
        _StringTable.ReviewAdExtension: _EntityInfo(lambda: BulkReviewAdExtension()),
        _StringTable.AccountReviewAdExtension: _EntityInfo(lambda: BulkAccountReviewAdExtension()),
        _StringTable.CampaignReviewAdExtension: _EntityInfo(lambda: BulkCampaignReviewAdExtension()),
        _StringTable.AdGroupReviewAdExtension: _EntityInfo(lambda: BulkAdGroupReviewAdExtension()),
        _StringTable.LocationAdExtension: _EntityInfo(lambda: BulkLocationAdExtension()),
        _StringTable.AccountLocationAdExtension: _EntityInfo(lambda: BulkAccountLocationAdExtension()),
        _StringTable.CampaignLocationAdExtension: _EntityInfo(lambda: BulkCampaignLocationAdExtension()),
        _StringTable.AppAdExtension: _EntityInfo(lambda: BulkAppAdExtension()),
        _StringTable.AccountAppAdExtension: _EntityInfo(lambda: BulkAccountAppAdExtension()),
        _StringTable.CampaignAppAdExtension: _EntityInfo(lambda: BulkCampaignAppAdExtension()),
        _StringTable.AdGroupAppAdExtension: _EntityInfo(lambda: BulkAdGroupAppAdExtension()),
        _StringTable.StructuredSnippetAdExtension: _EntityInfo(lambda: BulkStructuredSnippetAdExtension()),
        _StringTable.AccountStructuredSnippetAdExtension: _EntityInfo(lambda: BulkAccountStructuredSnippetAdExtension()),
        _StringTable.CampaignStructuredSnippetAdExtension: _EntityInfo(lambda: BulkCampaignStructuredSnippetAdExtension()),
        _StringTable.AdGroupStructuredSnippetAdExtension: _EntityInfo(lambda: BulkAdGroupStructuredSnippetAdExtension()),
        _StringTable.SitelinkAdExtension: _EntityInfo(lambda: BulkSitelinkAdExtension()),
        _StringTable.AccountSitelinkAdExtension: _EntityInfo(lambda: BulkAccountSitelinkAdExtension()),
        _StringTable.CampaignSitelinkAdExtension: _EntityInfo(lambda: BulkCampaignSitelinkAdExtension()),
        _StringTable.AdGroupSitelinkAdExtension: _EntityInfo(lambda: BulkAdGroupSitelinkAdExtension()),
        _StringTable.PriceAdExtension: _EntityInfo(lambda: BulkPriceAdExtension()),
        _StringTable.AccountPriceAdExtension: _EntityInfo(lambda: BulkAccountPriceAdExtension()),
        _StringTable.CampaignPriceAdExtension: _EntityInfo(lambda: BulkCampaignPriceAdExtension()),
        _StringTable.AdGroupPriceAdExtension: _EntityInfo(lambda: BulkAdGroupPriceAdExtension()),
        _StringTable.PromotionAdExtension: _EntityInfo(lambda: BulkPromotionAdExtension()),
        _StringTable.AccountPromotionAdExtension: _EntityInfo(lambda: BulkAccountPromotionAdExtension()),
        _StringTable.CampaignPromotionAdExtension: _EntityInfo(lambda: BulkCampaignPromotionAdExtension()),
        _StringTable.AdGroupPromotionAdExtension: _EntityInfo(lambda: BulkAdGroupPromotionAdExtension()),
        _StringTable.ProductAd: _EntityInfo(lambda: BulkProductAd()),
        _StringTable.TextAd: _EntityInfo(lambda: BulkTextAd()),
        _StringTable.AppInstallAd: _EntityInfo(lambda: BulkAppInstallAd()),
        _StringTable.ExpandedTextAd: _EntityInfo(lambda: BulkExpandedTextAd()),
        _StringTable.DynamicSearchAd: _EntityInfo(lambda: BulkDynamicSearchAd()),
        _StringTable.ResponsiveAd: _EntityInfo(lambda: BulkResponsiveAd()),
        _StringTable.ResponsiveSearchAd: _EntityInfo(lambda: BulkResponsiveSearchAd()),
        "Campaign Negative Site": _EntityInfo(
            lambda: BulkCampaignNegativeSite(),
            _StringTable.Website,
            lambda: _BulkCampaignNegativeSitesIdentifier()
        ),
        "Ad Group Negative Site": _EntityInfo(
            lambda: BulkAdGroupNegativeSite(),
            _StringTable.Website,
            lambda: _BulkAdGroupNegativeSitesIdentifier()
        ),

        _StringTable.NegativeKeywordList: _EntityInfo(lambda: BulkNegativeKeywordList()),
        _StringTable.ListNegativeKeyword: _EntityInfo(lambda: BulkSharedNegativeKeyword()),
        _StringTable.CampaignNegativeKeywordList: _EntityInfo(lambda: BulkCampaignNegativeKeywordList()),
        _StringTable.CampaignNegativeKeyword: _EntityInfo(lambda: BulkCampaignNegativeKeyword()),
        _StringTable.AdGroupNegativeKeyword: _EntityInfo(lambda: BulkAdGroupNegativeKeyword()),
        'Campaign Product Scope': _EntityInfo(lambda : BulkCampaignProductScope()),
        'Ad Group Product Partition': _EntityInfo(lambda : BulkAdGroupProductPartition()),
        'Ad Group Hotel Listing Group': _EntityInfo(lambda : BulkAdGroupHotelListingGroup()),
        _StringTable.RemarketingList: _EntityInfo(lambda : BulkRemarketingList()),
        _StringTable.AdGroupRemarketingListAssociation: _EntityInfo(lambda : BulkAdGroupRemarketingListAssociation()),
        _StringTable.AdGroupNegativeRemarketingListAssociation: _EntityInfo(lambda : BulkAdGroupNegativeRemarketingListAssociation()),
        _StringTable.CampaignRemarketingListAssociation: _EntityInfo(lambda : BulkCampaignRemarketingListAssociation()),
        _StringTable.CampaignNegativeRemarketingListAssociation: _EntityInfo(lambda : BulkCampaignNegativeRemarketingListAssociation()),
        _StringTable.CampaignNegativeStoreCriterion: _EntityInfo(lambda : BulkCampaignNegativeStoreCriterion()),
        _StringTable.CustomAudience: _EntityInfo(lambda : BulkCustomAudience()),
        _StringTable.AdGroupCustomAudienceAssociation: _EntityInfo(lambda : BulkAdGroupCustomAudienceAssociation()),
        _StringTable.AdGroupNegativeCustomAudienceAssociation: _EntityInfo(lambda : BulkAdGroupNegativeCustomAudienceAssociation()),
        _StringTable.CampaignCustomAudienceAssociation: _EntityInfo(lambda : BulkCampaignCustomAudienceAssociation()),
        _StringTable.CampaignNegativeCustomAudienceAssociation: _EntityInfo(lambda : BulkCampaignNegativeCustomAudienceAssociation()),
        _StringTable.InMarketAudience: _EntityInfo(lambda : BulkInMarketAudience()),
        _StringTable.AdGroupInMarketAudienceAssociation: _EntityInfo(lambda : BulkAdGroupInMarketAudienceAssociation()),
        _StringTable.AdGroupNegativeInMarketAudienceAssociation: _EntityInfo(lambda : BulkAdGroupNegativeInMarketAudienceAssociation()),
        _StringTable.CampaignInMarketAudienceAssociation: _EntityInfo(lambda : BulkCampaignInMarketAudienceAssociation()),
        _StringTable.CampaignNegativeInMarketAudienceAssociation: _EntityInfo(lambda : BulkCampaignNegativeInMarketAudienceAssociation()),
        'Campaign Negative Dynamic Search Ad Target': _EntityInfo(lambda: BulkCampaignNegativeDynamicSearchAdTarget()),
        'Ad Group Dynamic Search Ad Target': _EntityInfo(lambda: BulkAdGroupDynamicSearchAdTarget()),
        'Ad Group Negative Dynamic Search Ad Target': _EntityInfo(lambda: BulkAdGroupNegativeDynamicSearchAdTarget()),
        'Ad Group Age Criterion': _EntityInfo(lambda: BulkAdGroupAgeCriterion()),
        'Ad Group DayTime Criterion': _EntityInfo(lambda: BulkAdGroupDayTimeCriterion()),
        'Ad Group DeviceOS Criterion': _EntityInfo(lambda: BulkAdGroupDeviceCriterion()),
        'Ad Group Gender Criterion': _EntityInfo(lambda: BulkAdGroupGenderCriterion()),
        'Ad Group Location Criterion': _EntityInfo(lambda: BulkAdGroupLocationCriterion()),
        'Ad Group Location Intent Criterion': _EntityInfo(lambda: BulkAdGroupLocationIntentCriterion()),
        'Ad Group Negative Location Criterion': _EntityInfo(lambda: BulkAdGroupNegativeLocationCriterion()),
        'Ad Group Radius Criterion': _EntityInfo(lambda: BulkAdGroupRadiusCriterion()),

        _StringTable.AdGroupAdvanceBookingWindowCriterion: _EntityInfo(lambda: BulkAdGroupHotelAdvanceBookingWindowCriterion()),
        _StringTable.AdGroupCheckInDateCriterion: _EntityInfo(lambda: BulkAdGroupHotelCheckInDateCriterion()),
        _StringTable.AdGroupCheckInDayCriterion: _EntityInfo(lambda: BulkAdGroupHotelCheckInDayCriterion()),
        _StringTable.AdGroupHotelDateSelectionTypeCriterion: _EntityInfo(lambda: BulkAdGroupHotelDateSelectionTypeCriterion()),
        _StringTable.AdGroupLengthOfStayCriterion: _EntityInfo(lambda: BulkAdGroupHotelLengthOfStayCriterion()),
        _StringTable.CampaignAgeCriterion: _EntityInfo(lambda: BulkCampaignAgeCriterion()),
        _StringTable.CampaignDayTimeCriterion: _EntityInfo(lambda: BulkCampaignDayTimeCriterion()),
        _StringTable.CampaignDeviceOSCriterion: _EntityInfo(lambda: BulkCampaignDeviceCriterion()),
        _StringTable.CampaignGenderCriterion: _EntityInfo(lambda: BulkCampaignGenderCriterion()),
        _StringTable.CampaignLocationCriterion: _EntityInfo(lambda: BulkCampaignLocationCriterion()),
        _StringTable.CampaignLocationIntentCriterion: _EntityInfo(lambda: BulkCampaignLocationIntentCriterion()),
        _StringTable.CampaignNegativeLocationCriterion: _EntityInfo(lambda: BulkCampaignNegativeLocationCriterion()),
        _StringTable.CampaignRadiusCriterion: _EntityInfo(lambda: BulkCampaignRadiusCriterion()),
        _StringTable.CampaignCompanyNameCriterion: _EntityInfo(lambda: BulkCampaignCompanyNameCriterion()),
        _StringTable.CampaignJobFunctionCriterion: _EntityInfo(lambda: BulkCampaignJobFunctionCriterion()),
        _StringTable.CampaignIndustryCriterion: _EntityInfo(lambda: BulkCampaignIndustryCriterion()),
        _StringTable.CampaignDealCriterion: _EntityInfo(lambda: BulkCampaignDealCriterion()),
        _StringTable.CombinedList: _EntityInfo(lambda: BulkCombinedList()),
        _StringTable.CustomerList: _EntityInfo(lambda: BulkCustomerList()),
        _StringTable.CustomerListItem: _EntityInfo(lambda: BulkCustomerListItem()),
        _StringTable.ProductAudience: _EntityInfo(lambda: BulkProductAudience()),
        _StringTable.AdGroupProductAudienceAssociation: _EntityInfo(lambda: BulkAdGroupProductAudienceAssociation()),
        _StringTable.AdGroupNegativeProductAudienceAssociation: _EntityInfo(lambda: BulkAdGroupNegativeProductAudienceAssociation()),
        _StringTable.AdGroupCombinedListAssociation: _EntityInfo(lambda: BulkAdGroupCombinedListAssociation()),
        _StringTable.AdGroupCustomerListAssociation: _EntityInfo(lambda: BulkAdGroupCustomerListAssociation()),
        _StringTable.AdGroupNegativeCombinedListAssociation: _EntityInfo(lambda: BulkAdGroupNegativeCombinedListAssociation()),
        _StringTable.AdGroupNegativeCustomerListAssociation: _EntityInfo(lambda: BulkAdGroupNegativeCustomerListAssociation()),
        _StringTable.CampaignProductAudienceAssociation: _EntityInfo(lambda: BulkCampaignProductAudienceAssociation()),
        _StringTable.CampaignNegativeProductAudienceAssociation: _EntityInfo(lambda: BulkCampaignNegativeProductAudienceAssociation()),
        _StringTable.CampaignCombinedListAssociation: _EntityInfo(lambda: BulkCampaignCombinedListAssociation()),
        _StringTable.CampaignNegativeCombinedListAssociation: _EntityInfo(lambda: BulkCampaignNegativeCombinedListAssociation()),
        _StringTable.CampaignCustomerListAssociation: _EntityInfo(lambda: BulkCampaignCustomerListAssociation()),
        _StringTable.CampaignNegativeCustomerListAssociation: _EntityInfo(lambda: BulkCampaignNegativeCustomerListAssociation()),
        _StringTable.ImpressionBasedRemarketingList: _EntityInfo(lambda: BulkImpressionBasedRemarketingList()),
        _StringTable.CampaignImpressionBasedRemarketingListAssociation: _EntityInfo(lambda: BulkCampaignImpressionBasedRemarketingListAssociation()),
        _StringTable.CampaignNegativeImpressionBasedRemarketingListAssociation: _EntityInfo(lambda: BulkCampaignNegativeImpressionBasedRemarketingListAssociation()),
        _StringTable.AdGroupImpressionBasedRemarketingListAssociation: _EntityInfo(lambda: BulkAdGroupImpressionBasedRemarketingListAssociation()),
        _StringTable.AdGroupNegativeImpressionBasedRemarketingListAssociation: _EntityInfo(lambda: BulkAdGroupNegativeImpressionBasedRemarketingListAssociation()),
        _StringTable.AdGroupIndustryCriterion: _EntityInfo(lambda: BulkAdGroupIndustryCriterion()),
        _StringTable.AdGroupCompanyNameCriterion: _EntityInfo(lambda: BulkAdGroupCompanyNameCriterion()),
        _StringTable.AdGroupJobFunctionCriterion: _EntityInfo(lambda: BulkAdGroupJobFunctionCriterion()),
        _StringTable.AdGroupNegativeAgeCriterion: _EntityInfo(lambda: BulkAdGroupNegativeAgeCriterion()),
        _StringTable.AdGroupNegativeCompanyNameCriterion: _EntityInfo(lambda: BulkAdGroupNegativeCompanyNameCriterion()),
        _StringTable.AdGroupNegativeGenderCriterion: _EntityInfo(lambda: BulkAdGroupNegativeGenderCriterion()),
        _StringTable.AdGroupNegativeIndustryCriterion: _EntityInfo(lambda: BulkAdGroupNegativeIndustryCriterion()),
        _StringTable.AdGroupNegativeJobFunctionCriterion: _EntityInfo(lambda: BulkAdGroupNegativeJobFunctionCriterion()),
        _StringTable.AdGroupGenreCriterion: _EntityInfo(lambda: BulkAdGroupGenreCriterion()),
        _StringTable.Label: _EntityInfo(lambda: BulkLabel()),
        _StringTable.CampaignLabel: _EntityInfo(lambda: BulkCampaignLabel()),
        _StringTable.AdGroupLabel: _EntityInfo(lambda: BulkAdGroupLabel()),
        _StringTable.KeywordLabel: _EntityInfo(lambda: BulkKeywordLabel()),
        _StringTable.AppInstallAdLabel: _EntityInfo(lambda: BulkAppInstallAdLabel()),
        _StringTable.DynamicSearchAdLabel: _EntityInfo(lambda: BulkDynamicSearchAdLabel()),
        _StringTable.ExpandedTextAdLabel: _EntityInfo(lambda: BulkExpandedTextAdLabel()),
        _StringTable.ProductAdLabel: _EntityInfo(lambda: BulkProductAdLabel()),
        _StringTable.ResponsiveAdLabel: _EntityInfo(lambda: BulkResponsiveAdLabel()),
        _StringTable.ResponsiveSearchAdLabel: _EntityInfo(lambda: BulkResponsiveSearchAdLabel()),
        _StringTable.OfflineConversion: _EntityInfo(lambda: BulkOfflineConversion()),
        _StringTable.OnlineConversionAdjustment: _EntityInfo(lambda: BulkOnlineConversionAdjustment()),
        _StringTable.SimilarRemarketingList: _EntityInfo(lambda: BulkSimilarRemarketingList()),
        _StringTable.AdGroupSimilarRemarketingListAssociation: _EntityInfo(lambda: BulkAdGroupSimilarRemarketingListAssociation()),
        _StringTable.AdGroupNegativeSimilarRemarketingListAssociation: _EntityInfo(lambda: BulkAdGroupNegativeSimilarRemarketingListAssociation()),
        _StringTable.CampaignSimilarRemarketingListAssociation: _EntityInfo(lambda: BulkCampaignSimilarRemarketingListAssociation()),
        _StringTable.CampaignNegativeSimilarRemarketingListAssociation: _EntityInfo(lambda: BulkCampaignNegativeSimilarRemarketingListAssociation()),
        _StringTable.ActionAdExtension: _EntityInfo(lambda: BulkActionAdExtension()),
        _StringTable.AccountActionAdExtension: _EntityInfo(lambda: BulkAccountActionAdExtension()),
        _StringTable.AdGroupActionAdExtension: _EntityInfo(lambda: BulkAdGroupActionAdExtension()),
        _StringTable.CampaignActionAdExtension: _EntityInfo(lambda: BulkCampaignActionAdExtension()),
        _StringTable.Experiment: _EntityInfo(lambda: BulkExperiment()),
        _StringTable.Feed: _EntityInfo(lambda: BulkFeed()),
        _StringTable.FeedItem: _EntityInfo(lambda: BulkFeedItem()),
        _StringTable.CampaignConversionGoal: _EntityInfo(lambda: BulkCampaignConversionGoal()),
        _StringTable.AdCustomizerAttribute: _EntityInfo(lambda: BulkAdCustomizerAttribute()),
        _StringTable.AdCustomizerCampaign: _EntityInfo(lambda: BulkCampaignAdCustomizerAttribute()),
        _StringTable.AdCustomizerAdGroup: _EntityInfo(lambda: BulkAdGroupAdCustomizerAttribute()),
        _StringTable.AdCustomizerKeyword: _EntityInfo(lambda: BulkKeywordAdCustomizerAttribute()),
        _StringTable.AssetGroup: _EntityInfo(lambda: BulkAssetGroup()),
        _StringTable.AudienceGroup: _EntityInfo(lambda: BulkAudienceGroup()),
        _StringTable.CampaignNegativeWebpage: _EntityInfo(lambda: BulkCampaignNegativeWebpage()),
        _StringTable.AssetGroupListingGroup: _EntityInfo(lambda: BulkAssetGroupListingGroup()),
        _StringTable.AudienceGroupAssetGroupAssociation: _EntityInfo(lambda: BulkAudienceGroupAssetGroupAssociation()),
        _StringTable.SeasonalityAdjustment: _EntityInfo(lambda: BulkSeasonalityAdjustment()),
        _StringTable.DataExclusion: _EntityInfo(lambda: BulkDataExclusion()),
        'Account Negative Keyword List': _EntityInfo(lambda: BulkAccountNegativeKeywordList()),
        'Account Negative Keyword List Association': _EntityInfo(lambda: BulkAccountNegativeKeywordListAssociation()),
        'Account Shared Negative Keyword': _EntityInfo(lambda: BulkAccountSharedNegativeKeyword()),
        _StringTable.EventGoal: _EntityInfo(lambda: BulkEventGoal()),
        _StringTable.AppInstallGoal: _EntityInfo(lambda: BulkAppInstallGoal()),
        _StringTable.DurationGoal: _EntityInfo(lambda: BulkDurationGoal()),
        _StringTable.OfflineConversionGoal: _EntityInfo(lambda: BulkOfflineConversionGoal()),
        _StringTable.UrlGoal: _EntityInfo(lambda: BulkUrlGoal()),
        _StringTable.InStoreTransactionGoal: _EntityInfo(lambda: BulkInStoreTransactionGoal()),
        _StringTable.PagesViewedPerVisitGoal: _EntityInfo(lambda: BulkPagesViewedPerVisitGoal()),
        _StringTable.InStoreVisitGoal: _EntityInfo(lambda: BulkInStoreVisitGoal()),
        _StringTable.ProductGoal: _EntityInfo(lambda: BulkProductGoal()),
        _StringTable.AssetGroupSearchTheme: _EntityInfo(lambda: BulkAssetGroupSearchTheme()),
        _StringTable.BrandList: _EntityInfo(lambda: BulkBrandList()),
        _StringTable.BrandItem: _EntityInfo(lambda: BulkBrandItem()),
        _StringTable.CampaignBrandList: _EntityInfo(lambda: BulkCampaignBrandListAssociation()),
        _StringTable.AssetGroupUrlTarget: _EntityInfo(lambda: BulkAssetGroupUrlTarget()),
        _StringTable.NewCustomerAcquisitionGoal: _EntityInfo(lambda: BulkNewCustomerAcquisitionGoal()),
    }

    ADDITIONAL_OBJECT_MAP = {
        'Format Version': lambda: _FormatVersion(),
        'Keyword Best Position Bid': lambda: BulkKeywordBestPositionBid(),
        'Keyword Main Line Bid': lambda: BulkKeywordMainLineBid(),
        'Keyword First Page Bid': lambda: BulkKeywordFirstPageBid(),
    }

    TYPE_REVERSE_MAP = {}
    TARGET_IDENTIFIER_TYPE_REVERSE_MAP = {}

    @staticmethod
    def create_bulk_object(row_values):
        type_column = row_values[_StringTable.Type]

        if type_column.endswith('Error'):
            return BulkError()
        elif type_column in _BulkObjectFactory.ADDITIONAL_OBJECT_MAP:
            return _BulkObjectFactory.ADDITIONAL_OBJECT_MAP[type_column]()
        elif type_column in _BulkObjectFactory.INDIVIDUAL_ENTITY_MAP:
            info = _BulkObjectFactory.INDIVIDUAL_ENTITY_MAP[type_column]
            if row_values[_StringTable.Status] == 'Deleted' \
                    and info.delete_all_column_name \
                    and not row_values[info.delete_all_column_name]:
                return info.create_identifier_func()
            return info.create_func()
        else:
            return UnknownBulkEntity()

    @staticmethod
    def get_bulk_row_type(bulk_object):
        if isinstance(bulk_object, BulkError):
            return '{0} Error'.format(_BulkObjectFactory.get_bulk_row_type(bulk_object.entity))
        return _BulkObjectFactory.TYPE_REVERSE_MAP[type(bulk_object)]


for (k, v) in _BulkObjectFactory.INDIVIDUAL_ENTITY_MAP.items():
    _BulkObjectFactory.TYPE_REVERSE_MAP[type(v.create_func())] = k

    if v.create_identifier_func is not None:
        identifier = v.create_identifier_func()
        _BulkObjectFactory.TYPE_REVERSE_MAP[type(identifier)] = k



# coding: utf-8

"""
    AdInsightService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
import pprint
from pydantic import BaseModel, ConfigDict, Field, StrictStr, ValidationError, field_validator
from typing import Any, List, Optional
from openapi_client.models.adinsight.auction_segment import AuctionSegment
from openapi_client.models.adinsight.auction_segment_search_parameter import AuctionSegmentSearchParameter
from openapi_client.models.adinsight.category_search_parameter import CategorySearchParameter
from openapi_client.models.adinsight.competition_level import CompetitionLevel
from openapi_client.models.adinsight.competition_search_parameter import CompetitionSearchParameter
from openapi_client.models.adinsight.date_range_search_parameter import DateRangeSearchParameter
from openapi_client.models.adinsight.day_month_and_year import DayMonthAndYear
from openapi_client.models.adinsight.device_criterion import DeviceCriterion
from openapi_client.models.adinsight.device_search_parameter import DeviceSearchParameter
from openapi_client.models.adinsight.exclude_account_keywords_search_parameter import ExcludeAccountKeywordsSearchParameter
from openapi_client.models.adinsight.idea_text_search_parameter import IdeaTextSearchParameter
from openapi_client.models.adinsight.impression_share_search_parameter import ImpressionShareSearchParameter
from openapi_client.models.adinsight.keyword import Keyword
from openapi_client.models.adinsight.language_criterion import LanguageCriterion
from openapi_client.models.adinsight.language_search_parameter import LanguageSearchParameter
from openapi_client.models.adinsight.location_criterion import LocationCriterion
from openapi_client.models.adinsight.location_search_parameter import LocationSearchParameter
from openapi_client.models.adinsight.network_criterion import NetworkCriterion
from openapi_client.models.adinsight.network_search_parameter import NetworkSearchParameter
from openapi_client.models.adinsight.query_search_parameter import QuerySearchParameter
from openapi_client.models.adinsight.search_volume_search_parameter import SearchVolumeSearchParameter
from openapi_client.models.adinsight.suggested_bid_search_parameter import SuggestedBidSearchParameter
from openapi_client.models.adinsight.url_search_parameter import UrlSearchParameter
from pydantic import StrictStr, Field
from typing import Union, List, Set, Optional, Dict
from typing_extensions import Literal, Self

SearchParameter_ONE_OF_SCHEMAS = ["AuctionSegmentSearchParameter", "CategorySearchParameter", "CompetitionSearchParameter", "DateRangeSearchParameter", "DeviceSearchParameter", "ExcludeAccountKeywordsSearchParameter", "IdeaTextSearchParameter", "ImpressionShareSearchParameter", "LanguageSearchParameter", "LocationSearchParameter", "NetworkSearchParameter", "QuerySearchParameter", "SearchVolumeSearchParameter", "SuggestedBidSearchParameter", "UrlSearchParameter"]

class SearchParameter(BaseModel):
    """
    SearchParameter
    """
    # data type: AuctionSegmentSearchParameter
    oneof_schema_auction_segment_search_parameter_validator: Optional[AuctionSegmentSearchParameter] = None
    # data type: CategorySearchParameter
    oneof_schema_category_search_parameter_validator: Optional[CategorySearchParameter] = None
    # data type: CompetitionSearchParameter
    oneof_schema_competition_search_parameter_validator: Optional[CompetitionSearchParameter] = None
    # data type: DateRangeSearchParameter
    oneof_schema_date_range_search_parameter_validator: Optional[DateRangeSearchParameter] = None
    # data type: DeviceSearchParameter
    oneof_schema_device_search_parameter_validator: Optional[DeviceSearchParameter] = None
    # data type: ExcludeAccountKeywordsSearchParameter
    oneof_schema_exclude_account_keywords_search_parameter_validator: Optional[ExcludeAccountKeywordsSearchParameter] = None
    # data type: IdeaTextSearchParameter
    oneof_schema_idea_text_search_parameter_validator: Optional[IdeaTextSearchParameter] = None
    # data type: ImpressionShareSearchParameter
    oneof_schema_impression_share_search_parameter_validator: Optional[ImpressionShareSearchParameter] = None
    # data type: LanguageSearchParameter
    oneof_schema_language_search_parameter_validator: Optional[LanguageSearchParameter] = None
    # data type: LocationSearchParameter
    oneof_schema_location_search_parameter_validator: Optional[LocationSearchParameter] = None
    # data type: NetworkSearchParameter
    oneof_schema_network_search_parameter_validator: Optional[NetworkSearchParameter] = None
    # data type: QuerySearchParameter
    oneof_schema_query_search_parameter_validator: Optional[QuerySearchParameter] = None
    # data type: SearchVolumeSearchParameter
    oneof_schema_search_volume_search_parameter_validator: Optional[SearchVolumeSearchParameter] = None
    # data type: SuggestedBidSearchParameter
    oneof_schema_suggested_bid_search_parameter_validator: Optional[SuggestedBidSearchParameter] = None
    # data type: UrlSearchParameter
    oneof_schema_url_search_parameter_validator: Optional[UrlSearchParameter] = None
    actual_instance: Optional[Union[AuctionSegmentSearchParameter, CategorySearchParameter, CompetitionSearchParameter, DateRangeSearchParameter, DeviceSearchParameter, ExcludeAccountKeywordsSearchParameter, IdeaTextSearchParameter, ImpressionShareSearchParameter, LanguageSearchParameter, LocationSearchParameter, NetworkSearchParameter, QuerySearchParameter, SearchVolumeSearchParameter, SuggestedBidSearchParameter, UrlSearchParameter]] = None
    one_of_schemas: Set[str] = { "AuctionSegmentSearchParameter", "CategorySearchParameter", "CompetitionSearchParameter", "DateRangeSearchParameter", "DeviceSearchParameter", "ExcludeAccountKeywordsSearchParameter", "IdeaTextSearchParameter", "ImpressionShareSearchParameter", "LanguageSearchParameter", "LocationSearchParameter", "NetworkSearchParameter", "QuerySearchParameter", "SearchVolumeSearchParameter", "SuggestedBidSearchParameter", "UrlSearchParameter" }

    model_config = ConfigDict(
        validate_assignment=True,
        protected_namespaces=(),
    )

    discriminator_value_class_map: Dict[str, str] = {
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError("If a position argument is used, only 1 is allowed to set `actual_instance`")
            if kwargs:
                raise ValueError("If a position argument is used, keyword arguments cannot be used.")
            super().__init__(actual_instance=args[0])
        else:
            super().__init__(**kwargs)

    @field_validator('actual_instance')
    def actual_instance_must_validate_oneof(cls, v):
        if v is None:
            return v

        instance = SearchParameter.model_construct()
        error_messages = []
        match = 0
        # validate data type: AuctionSegmentSearchParameter
        if not isinstance(v, AuctionSegmentSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `AuctionSegmentSearchParameter`")
        else:
            match += 1
        # validate data type: CategorySearchParameter
        if not isinstance(v, CategorySearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CategorySearchParameter`")
        else:
            match += 1
        # validate data type: CompetitionSearchParameter
        if not isinstance(v, CompetitionSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `CompetitionSearchParameter`")
        else:
            match += 1
        # validate data type: DateRangeSearchParameter
        if not isinstance(v, DateRangeSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DateRangeSearchParameter`")
        else:
            match += 1
        # validate data type: DeviceSearchParameter
        if not isinstance(v, DeviceSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `DeviceSearchParameter`")
        else:
            match += 1
        # validate data type: ExcludeAccountKeywordsSearchParameter
        if not isinstance(v, ExcludeAccountKeywordsSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ExcludeAccountKeywordsSearchParameter`")
        else:
            match += 1
        # validate data type: IdeaTextSearchParameter
        if not isinstance(v, IdeaTextSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `IdeaTextSearchParameter`")
        else:
            match += 1
        # validate data type: ImpressionShareSearchParameter
        if not isinstance(v, ImpressionShareSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `ImpressionShareSearchParameter`")
        else:
            match += 1
        # validate data type: LanguageSearchParameter
        if not isinstance(v, LanguageSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LanguageSearchParameter`")
        else:
            match += 1
        # validate data type: LocationSearchParameter
        if not isinstance(v, LocationSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `LocationSearchParameter`")
        else:
            match += 1
        # validate data type: NetworkSearchParameter
        if not isinstance(v, NetworkSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `NetworkSearchParameter`")
        else:
            match += 1
        # validate data type: QuerySearchParameter
        if not isinstance(v, QuerySearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `QuerySearchParameter`")
        else:
            match += 1
        # validate data type: SearchVolumeSearchParameter
        if not isinstance(v, SearchVolumeSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `SearchVolumeSearchParameter`")
        else:
            match += 1
        # validate data type: SuggestedBidSearchParameter
        if not isinstance(v, SuggestedBidSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `SuggestedBidSearchParameter`")
        else:
            match += 1
        # validate data type: UrlSearchParameter
        if not isinstance(v, UrlSearchParameter):
            error_messages.append(f"Error! Input type `{type(v)}` is not `UrlSearchParameter`")
        else:
            match += 1
        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when setting `actual_instance` in SearchParameter with oneOf schemas: AuctionSegmentSearchParameter, CategorySearchParameter, CompetitionSearchParameter, DateRangeSearchParameter, DeviceSearchParameter, ExcludeAccountKeywordsSearchParameter, IdeaTextSearchParameter, ImpressionShareSearchParameter, LanguageSearchParameter, LocationSearchParameter, NetworkSearchParameter, QuerySearchParameter, SearchVolumeSearchParameter, SuggestedBidSearchParameter, UrlSearchParameter. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when setting `actual_instance` in SearchParameter with oneOf schemas: AuctionSegmentSearchParameter, CategorySearchParameter, CompetitionSearchParameter, DateRangeSearchParameter, DeviceSearchParameter, ExcludeAccountKeywordsSearchParameter, IdeaTextSearchParameter, ImpressionShareSearchParameter, LanguageSearchParameter, LocationSearchParameter, NetworkSearchParameter, QuerySearchParameter, SearchVolumeSearchParameter, SuggestedBidSearchParameter, UrlSearchParameter. Details: " + ", ".join(error_messages))
        else:
            return v

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        return cls.from_json(json.dumps(obj))

    @classmethod
    def from_json(cls, json_str: Optional[str]) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        if json_str is None:
            return instance

        error_messages = []
        match = 0

        # use oneOf discriminator to lookup the data type
        _data_type = json.loads(json_str).get("Type")
        if not _data_type:
            raise ValueError("Failed to lookup data type from the field `Type` in the input.")

		# check if data type is `AuctionSegmentSearchParameter`
        if _data_type == "AuctionSegmentSearchParameter":
            instance.actual_instance = AuctionSegmentSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `CategorySearchParameter`
        if _data_type == "CategorySearchParameter":
            instance.actual_instance = CategorySearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `CompetitionSearchParameter`
        if _data_type == "CompetitionSearchParameter":
            instance.actual_instance = CompetitionSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `DateRangeSearchParameter`
        if _data_type == "DateRangeSearchParameter":
            instance.actual_instance = DateRangeSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `DeviceSearchParameter`
        if _data_type == "DeviceSearchParameter":
            instance.actual_instance = DeviceSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `ExcludeAccountKeywordsSearchParameter`
        if _data_type == "ExcludeAccountKeywordsSearchParameter":
            instance.actual_instance = ExcludeAccountKeywordsSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `IdeaTextSearchParameter`
        if _data_type == "IdeaTextSearchParameter":
            instance.actual_instance = IdeaTextSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `ImpressionShareSearchParameter`
        if _data_type == "ImpressionShareSearchParameter":
            instance.actual_instance = ImpressionShareSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `LanguageSearchParameter`
        if _data_type == "LanguageSearchParameter":
            instance.actual_instance = LanguageSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `LocationSearchParameter`
        if _data_type == "LocationSearchParameter":
            instance.actual_instance = LocationSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `NetworkSearchParameter`
        if _data_type == "NetworkSearchParameter":
            instance.actual_instance = NetworkSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `QuerySearchParameter`
        if _data_type == "QuerySearchParameter":
            instance.actual_instance = QuerySearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `SearchVolumeSearchParameter`
        if _data_type == "SearchVolumeSearchParameter":
            instance.actual_instance = SearchVolumeSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `SuggestedBidSearchParameter`
        if _data_type == "SuggestedBidSearchParameter":
            instance.actual_instance = SuggestedBidSearchParameter.from_json(json_str)
            return instance
			
		# check if data type is `UrlSearchParameter`
        if _data_type == "UrlSearchParameter":
            instance.actual_instance = UrlSearchParameter.from_json(json_str)
            return instance
			

        # deserialize data into AuctionSegmentSearchParameter
        try:
            instance.actual_instance = AuctionSegmentSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into CategorySearchParameter
        try:
            instance.actual_instance = CategorySearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into CompetitionSearchParameter
        try:
            instance.actual_instance = CompetitionSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DateRangeSearchParameter
        try:
            instance.actual_instance = DateRangeSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into DeviceSearchParameter
        try:
            instance.actual_instance = DeviceSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ExcludeAccountKeywordsSearchParameter
        try:
            instance.actual_instance = ExcludeAccountKeywordsSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into IdeaTextSearchParameter
        try:
            instance.actual_instance = IdeaTextSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into ImpressionShareSearchParameter
        try:
            instance.actual_instance = ImpressionShareSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LanguageSearchParameter
        try:
            instance.actual_instance = LanguageSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into LocationSearchParameter
        try:
            instance.actual_instance = LocationSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into NetworkSearchParameter
        try:
            instance.actual_instance = NetworkSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into QuerySearchParameter
        try:
            instance.actual_instance = QuerySearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into SearchVolumeSearchParameter
        try:
            instance.actual_instance = SearchVolumeSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into SuggestedBidSearchParameter
        try:
            instance.actual_instance = SuggestedBidSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        # deserialize data into UrlSearchParameter
        try:
            instance.actual_instance = UrlSearchParameter.from_json(json_str)
            match += 1
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        if match > 1:
            # more than 1 match
            raise ValueError("Multiple matches found when deserializing the JSON string into SearchParameter with oneOf schemas: AuctionSegmentSearchParameter, CategorySearchParameter, CompetitionSearchParameter, DateRangeSearchParameter, DeviceSearchParameter, ExcludeAccountKeywordsSearchParameter, IdeaTextSearchParameter, ImpressionShareSearchParameter, LanguageSearchParameter, LocationSearchParameter, NetworkSearchParameter, QuerySearchParameter, SearchVolumeSearchParameter, SuggestedBidSearchParameter, UrlSearchParameter. Details: " + ", ".join(error_messages))
        elif match == 0:
            # no match
            raise ValueError("No match found when deserializing the JSON string into SearchParameter with oneOf schemas: AuctionSegmentSearchParameter, CategorySearchParameter, CompetitionSearchParameter, DateRangeSearchParameter, DeviceSearchParameter, ExcludeAccountKeywordsSearchParameter, IdeaTextSearchParameter, ImpressionShareSearchParameter, LanguageSearchParameter, LocationSearchParameter, NetworkSearchParameter, QuerySearchParameter, SearchVolumeSearchParameter, SuggestedBidSearchParameter, UrlSearchParameter. Details: " + ", ".join(error_messages))
        else:
            return instance

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(self.actual_instance.to_json):
            return self.actual_instance.to_json()
        else:
            return json.dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], AuctionSegmentSearchParameter, CategorySearchParameter, CompetitionSearchParameter, DateRangeSearchParameter, DeviceSearchParameter, ExcludeAccountKeywordsSearchParameter, IdeaTextSearchParameter, ImpressionShareSearchParameter, LanguageSearchParameter, LocationSearchParameter, NetworkSearchParameter, QuerySearchParameter, SearchVolumeSearchParameter, SuggestedBidSearchParameter, UrlSearchParameter]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(self.actual_instance.to_dict):
            return self.actual_instance.to_dict()
        else:
            # primitive type
            return self.actual_instance

    def to_str(self) -> str:
        """Returns the string representation of the actual instance"""
        return pprint.pformat(self.model_dump())

    def __getattr__(self, name):
        """Forward attribute access to actual_instance"""
        if self.actual_instance is not None and hasattr(self.actual_instance, name):
            return getattr(self.actual_instance, name)
        raise AttributeError(f"'{self.__class__.__name__}' object has no attribute '{name}'")

    def __setattr__(self, name, value):
        """Forward attribute setting to actual_instance"""
        if name in ['actual_instance', 'oneof_schema_auction_segment_search_parameter_validator', 'oneof_schema_category_search_parameter_validator', 'oneof_schema_competition_search_parameter_validator', 'oneof_schema_date_range_search_parameter_validator', 'oneof_schema_device_search_parameter_validator', 'oneof_schema_exclude_account_keywords_search_parameter_validator', 'oneof_schema_idea_text_search_parameter_validator', 'oneof_schema_impression_share_search_parameter_validator', 'oneof_schema_language_search_parameter_validator', 'oneof_schema_location_search_parameter_validator', 'oneof_schema_network_search_parameter_validator', 'oneof_schema_query_search_parameter_validator', 'oneof_schema_search_volume_search_parameter_validator', 'oneof_schema_suggested_bid_search_parameter_validator', 'oneof_schema_url_search_parameter_validator', 'one_of_schemas', 'model_config', 'discriminator_value_class_map']:
            super().__setattr__(name, value)
        elif self.actual_instance is not None and hasattr(self.actual_instance, name):
            setattr(self.actual_instance, name, value)
        else:
            super().__setattr__(name, value)



import os


class ReportingDownloadParameters:
    """ Describes the related parameters when downloading file from server.

    such as the file name and directory that you want to specify.
    """

    def __init__(self,
                 report_request=None,
                 result_file_directory=None,
                 result_file_name=None,
                 overwrite_result_file=False,
                 timeout_in_milliseconds=None):
        """
        :param report_request: the report request object, which derives from the base request report class
        :type report_request: ReportRequest
        :param result_file_directory: (optional) The directory where the file will be downloaded.
        :type result_file_directory: str
        :param result_file_name: (optional) The name of the download result file.
        :type result_file_name: str
        :param overwrite_result_file:
        :type overwrite_result_file: bool
        :param timeout_in_milliseconds: (optional) timeout for reporting download operations in milliseconds
        :type timeout_in_milliseconds: int
        :return:
        """

        self._report_request = report_request
        self._result_file_directory = result_file_directory
        self._result_file_name = result_file_name
        self._decompress_result_file = True
        if result_file_name is not None:
            _, ext = os.path.splitext(result_file_name)
            if ext == '.zip':
                self._decompress_result_file = False
        self._overwrite_result_file = overwrite_result_file
        self._timeout_in_milliseconds=timeout_in_milliseconds

    @property
    def result_file_directory(self):
        """ The directory where the reporting file will be downloaded.
        :rtype: str
        """

        return self._result_file_directory

    @property
    def result_file_name(self):
        """ The name of the download reporting file.
        :rtype: str
        """

        return self._result_file_name

    @property
    def overwrite_result_file(self):
        """ Whether the local result file should be overwritten if it already exists.
        :rtype: bool
        """

        return self._overwrite_result_file

    @result_file_directory.setter
    def result_file_directory(self, result_file_directory):
        self._result_file_directory = result_file_directory

    @result_file_name.setter
    def result_file_name(self, result_file_name):
        self._result_file_name = result_file_name

    @overwrite_result_file.setter
    def overwrite_result_file(self, overwrite):
        self._overwrite_result_file = overwrite

    @property
    def decompress_result_file(self):
        """ If need to decompress the result file after download.
        This property is determined by the result_file_name, by default will do decompression.
        if the result_file_name has the extension of '.zip' then do not do decompression.
        :rtype: bool
        """

        return self._decompress_result_file


    @property
    def report_request(self):
        """ The report request.
        :rtype: ReportRequest
        """

        return self._report_request


    @report_request.setter
    def report_request(self, value):
        self._report_request = value

    @property
    def timeout_in_milliseconds(self):
        return self._timeout_in_milliseconds

    @timeout_in_milliseconds.setter
    def timeout_in_milliseconds(self, value):
        self._timeout_in_milliseconds = value



# coding: utf-8

"""
    CampaignManagementService

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union, Set
from openapi_client.models.campaign.key_frame import KeyFrame
from openapi_client.models.campaign.video_recommendation_image_data import VideoRecommendationImageData
from openapi_client.models.campaign.video_recommendation_text_data import VideoRecommendationTextData
from typing_extensions import Self

class RecommendationScene(BaseModel):
    """
    RecommendationScene
    """ # noqa: E501
    image: Optional[StrictStr] = Field(default=None, alias="image")
    images: Optional[List[StrictStr]] = Field(default=None, alias="images")
    image_data: Optional[List[Optional[VideoRecommendationImageData]]] = Field(default=None, alias="imageData")
    text: Optional[List[StrictStr]] = Field(default=None, alias="text")
    text_data: Optional[List[Optional[VideoRecommendationTextData]]] = Field(default=None, alias="textData")
    logos: Optional[List[StrictStr]] = Field(default=None, alias="logos")
    logo_data: Optional[List[Optional[VideoRecommendationImageData]]] = Field(default=None, alias="logoData")
    key_frame: Optional[KeyFrame] = Field(default=None, alias="keyFrame")
    __properties: ClassVar[List[str]] = ["image", "images", "imageData", "text", "textData", "logos", "logoData", "keyFrame"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )
	

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in image_data (list)
        _items = []
        if self.image_data:
            for _item_image_data in self.image_data:
                if _item_image_data:
                    _items.append(_item_image_data.to_dict())
            _dict['ImageData'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in text_data (list)
        _items = []
        if self.text_data:
            for _item_text_data in self.text_data:
                if _item_text_data:
                    _items.append(_item_text_data.to_dict())
            _dict['TextData'] = _items
        # override the default output from pydantic by calling `to_dict()` of each item in logo_data (list)
        _items = []
        if self.logo_data:
            for _item_logo_data in self.logo_data:
                if _item_logo_data:
                    _items.append(_item_logo_data.to_dict())
            _dict['LogoData'] = _items
        # override the default output from pydantic by calling `to_dict()` of key_frame
        if self.key_frame:
            _dict['keyFrame'] = self.key_frame.to_dict()
        # set to None if image (nullable) is None
        # and model_fields_set contains the field
        if self.image is None and "image" in self.model_fields_set:
            _dict['image'] = None

        # set to None if images (nullable) is None
        # and model_fields_set contains the field
        if self.images is None and "images" in self.model_fields_set:
            _dict['images'] = None

        # set to None if image_data (nullable) is None
        # and model_fields_set contains the field
        if self.image_data is None and "image_data" in self.model_fields_set:
            _dict['imageData'] = None

        # set to None if text (nullable) is None
        # and model_fields_set contains the field
        if self.text is None and "text" in self.model_fields_set:
            _dict['text'] = None

        # set to None if text_data (nullable) is None
        # and model_fields_set contains the field
        if self.text_data is None and "text_data" in self.model_fields_set:
            _dict['textData'] = None

        # set to None if logos (nullable) is None
        # and model_fields_set contains the field
        if self.logos is None and "logos" in self.model_fields_set:
            _dict['logos'] = None

        # set to None if logo_data (nullable) is None
        # and model_fields_set contains the field
        if self.logo_data is None and "logo_data" in self.model_fields_set:
            _dict['logoData'] = None

        # set to None if key_frame (nullable) is None
        # and model_fields_set contains the field
        if self.key_frame is None and "key_frame" in self.model_fields_set:
            _dict['keyFrame'] = None

        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RecommendationScene from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "image": obj.get("Image") if obj.get("Image") is not None else None,
                        "images": obj.get("Images"),
                        "imageData": [VideoRecommendationImageData.from_dict(_item) for _item in obj["ImageData"]] if obj.get("ImageData") is not None else None,
                        "text": obj.get("Text"),
                        "textData": [VideoRecommendationTextData.from_dict(_item) for _item in obj["TextData"]] if obj.get("TextData") is not None else None,
                        "logos": obj.get("Logos"),
                        "logoData": [VideoRecommendationImageData.from_dict(_item) for _item in obj["LogoData"]] if obj.get("LogoData") is not None else None,
                        "keyFrame": KeyFrame.from_dict(obj["KeyFrame"]) if obj.get("KeyFrame") is not None else None
        })
        return _obj



from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13

from .common import _BulkAdExtensionBase
from .common import _BulkAdGroupAdExtensionAssociation
from .common import _BulkCampaignAdExtensionAssociation
from .common import _BulkAccountAdExtensionAssociation

from bingads.v13.internal.extensions import *

_SitelinkAdExtension = type(_CAMPAIGN_OBJECT_FACTORY_V13.create('SitelinkAdExtension'))


class BulkSitelinkAdExtension(_BulkAdExtensionBase):
    """ Represents a sitelink ad extension.

    This class exposes the :attr:`sitelink_ad_extension` property that can be read and written
    as fields of the Sitelink Ad Extension record in a bulk file.

    For more information, see Sitelink Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    def __init__(self, account_id=None, ad_extension=None):
        if ad_extension and not isinstance(ad_extension, _SitelinkAdExtension):
            raise ValueError('The type of ad_extension is: {0}, should be: {1}'.format(
                type(ad_extension),
                'SitelinkAdExtension'
            ))
        super(BulkSitelinkAdExtension, self).__init__(
            account_id=account_id,
            ad_extension=ad_extension
        )

    @property
    def sitelink_ad_extension(self):
        """ The sitelink ad extension.

        see Sitelink Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.
        """

        return self._ad_extension

    @sitelink_ad_extension.setter
    def sitelink_ad_extension(self, value):
        self._ad_extension = value

    _MAPPINGS = [
        _SimpleBulkMapping(
            header=_StringTable.SiteLinkDescription1,
            field_to_csv=lambda c: c.sitelink_ad_extension.Description1,
            csv_to_field=lambda c, v: setattr(c.sitelink_ad_extension, 'Description1', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.SiteLinkDescription2,
            field_to_csv=lambda c: c.sitelink_ad_extension.Description2,
            csv_to_field=lambda c, v: setattr(c.sitelink_ad_extension, 'Description2', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.DestinationUrl,
            field_to_csv=lambda c: bulk_optional_str(c.sitelink_ad_extension.DestinationUrl, c.sitelink_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.sitelink_ad_extension, 'DestinationUrl', v if v else '')
        ),
        _SimpleBulkMapping(
            header=_StringTable.SiteLinkDisplayText,
            field_to_csv=lambda c: c.sitelink_ad_extension.DisplayText,
            csv_to_field=lambda c, v: setattr(c.sitelink_ad_extension, 'DisplayText', v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.sitelink_ad_extension.FinalUrls, c.sitelink_ad_extension.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.sitelink_ad_extension.FinalUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalMobileUrl,
            field_to_csv=lambda c: field_to_csv_Urls(c.sitelink_ad_extension.FinalMobileUrls, c.sitelink_ad_extension.Id),
            csv_to_field=lambda c, v: csv_to_field_Urls(c.sitelink_ad_extension.FinalMobileUrls, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.TrackingTemplate,
            field_to_csv=lambda c: bulk_str(c.sitelink_ad_extension.TrackingUrlTemplate),
            csv_to_field=lambda c, v: setattr(c.sitelink_ad_extension, 'TrackingUrlTemplate', v if v else None)
        ),
        _SimpleBulkMapping(
            header=_StringTable.CustomParameter,
            field_to_csv=lambda c: field_to_csv_UrlCustomParameters(c.sitelink_ad_extension),
            csv_to_field=lambda c, v: csv_to_field_UrlCustomParameters(c.sitelink_ad_extension, v)
        ),
        _SimpleBulkMapping(
            header=_StringTable.FinalUrlSuffix,
            field_to_csv=lambda c: bulk_optional_str(c.sitelink_ad_extension.FinalUrlSuffix, c.sitelink_ad_extension.Id),
            csv_to_field=lambda c, v: setattr(c.sitelink_ad_extension, 'FinalUrlSuffix', v)
        )
    ]

    def process_mappings_from_row_values(self, row_values):
        self.sitelink_ad_extension = _CAMPAIGN_OBJECT_FACTORY_V13.create('SitelinkAdExtension')
        self.sitelink_ad_extension.Type = 'SitelinkAdExtension'
        super(BulkSitelinkAdExtension, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkSitelinkAdExtension._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        self._validate_property_not_null(self.sitelink_ad_extension, 'sitelink_ad_extension')
        super(BulkSitelinkAdExtension, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkSitelinkAdExtension._MAPPINGS)


class BulkAccountSitelinkAdExtension(_BulkAccountAdExtensionAssociation):
    """ Represents an account level sitelink ad extension.

    This class exposes properties that can be read and written
    as fields of the Account Sitelink Ad Extension record in a bulk file.

    For more information, see Account Sitelink Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkCampaignSitelinkAdExtension(_BulkCampaignAdExtensionAssociation):
    """ Represents a campaign level sitelink ad extension.

    This class exposes properties that can be read and written
    as fields of the Campaign Sitelink Ad Extension record in a bulk file.

    For more information, see Campaign Sitelink Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass

class BulkAdGroupSitelinkAdExtension(_BulkAdGroupAdExtensionAssociation):
    """ Represents an ad group level Sitelink ad extension.

    This class exposes properties that can be read and written
    as fields of the Ad Group Sitelink Ad Extension record in a bulk file.

    For more information, see Ad Group Sitelink Ad Extension at https://go.microsoft.com/fwlink/?linkid=846127.

    *See also:*

    * :class:`.BulkServiceManager`
    * :class:`.BulkOperation`
    * :class:`.BulkFileReader`
    * :class:`.BulkFileWriter`
    """

    pass



from bingads.v13.bulk.entities import *
from bingads.service_client import _CAMPAIGN_OBJECT_FACTORY_V13
from bingads.v13.bulk.entities.target_criterions.bulk_campaign_negative_criterion import BulkCampaignNegativeCriterion
from bingads.v13.internal.bulk.mappings import _SimpleBulkMapping
from bingads.v13.internal.bulk.string_table import _StringTable
from bingads.v13.internal.extensions import *


class BulkCampaignNegativeAudienceAssociation(BulkCampaignNegativeCriterion):
    """ Base class for all Campaign Negative Audience Association subclasses that can be read or written in a bulk file.

    *See also:*

    * :class:`.BulkCampaignNegativeCustomAudienceAssociation`
    * :class:`.BulkCampaignNegativeInMarketAudienceAssociation`
    * :class:`.BulkCampaignNegativeProductAudienceAssociation`
    * :class:`.BulkCampaignNegativeRemarketingListAssociation`
    * :class:`.BulkCampaignNegativeSimilarRemarketingListAssociation`
    """

    def __init__(self,
                 negative_campaign_criterion=None,
                 campaign_name=None,
                 audience_name=None):
        super(BulkCampaignNegativeAudienceAssociation, self).__init__(negative_campaign_criterion, campaign_name)
        self._audience_name = audience_name

    _MAPPINGS = [
        _SimpleBulkMapping(
            _StringTable.Audience,
            field_to_csv=lambda c: c.audience_name,
            csv_to_field=lambda c, v: setattr(c, 'audience_name', v)
        ),
        _SimpleBulkMapping(
            _StringTable.AudienceId,
            field_to_csv=lambda c: field_to_csv_CriterionAudienceId(c.negative_campaign_criterion),
            csv_to_field=lambda c, v: csv_to_field_CriterionAudienceId(c.negative_campaign_criterion, int(v) if v else None)
        ),
    ]

    @property
    def audience_name(self):
        """ Defines the name of the Audience

        :rtype: str
        """

        return self._audience_name

    @audience_name.setter
    def audience_name(self, audience_name):
        self._audience_name = audience_name
        
    def create_criterion(self):
        self._negative_campaign_criterion.Criterion = _CAMPAIGN_OBJECT_FACTORY_V13.create('AudienceCriterion')
        self._negative_campaign_criterion.Criterion.Type = 'AudienceCriterion'

    def process_mappings_from_row_values(self, row_values):
        super(BulkCampaignNegativeAudienceAssociation, self).process_mappings_from_row_values(row_values)
        row_values.convert_to_entity(self, BulkCampaignNegativeAudienceAssociation._MAPPINGS)

    def process_mappings_to_row_values(self, row_values, exclude_readonly_data):
        super(BulkCampaignNegativeAudienceAssociation, self).process_mappings_to_row_values(row_values, exclude_readonly_data)
        self.convert_to_values(row_values, BulkCampaignNegativeAudienceAssociation._MAPPINGS)
    
    def read_additional_data(self, stream_reader):
        super(BulkCampaignNegativeAudienceAssociation, self).read_additional_data(stream_reader)
